#!/usr/bin/python
""" PN CLI vrouter-create/vrouter-delete/vrouter-modify """

#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

DOCUMENTATION = '''
---
module: pn_vrouter
author: "Pluribus Networks (@amitsi)"
short_description: CLI command to create/delete/modify a vrouter.
deprecated:
  removed_in: 2.0.0  # was Ansible 2.12
  why: Doesn't support latest Pluribus Networks netvisor
  alternative: Latest modules will be pushed in Ansible future versions.
description:
  - Execute vrouter-create, vrouter-delete, vrouter-modify command.
  - Each fabric, cluster, standalone switch, or virtual network (VNET) can
    provide its tenants with a virtual router (vRouter) service that forwards
    traffic between networks and implements Layer 3 protocols.
  - C(vrouter-create) creates a new vRouter service.
  - C(vrouter-delete) deletes a vRouter service.
  - C(vrouter-modify) modifies a vRouter service.
options:
  pn_cliusername:
    description:
      - Provide login username if user is not root.
    required: False
  pn_clipassword:
    description:
      - Provide login password if user is not root.
    required: False
  pn_cliswitch:
    description:
      - Target switch(es) to run the CLI on.
    required: False
    default: 'local'
  state:
    description:
      - State the action to perform. Use 'present' to create vrouter,
        'absent' to delete vrouter and 'update' to modify vrouter.
    required: True
    choices: ['present', 'absent', 'update']
  pn_name:
    description:
      - Specify the name of the vRouter.
    required: true
  pn_vnet:
    description:
      - Specify the name of the VNET.
      - Required for vrouter-create.
  pn_service_type:
    description:
      - Specify if the vRouter is a dedicated or shared VNET service.
    choices: ['dedicated', 'shared']
  pn_service_state:
    description:
      -  Specify to enable or disable vRouter service.
    choices: ['enable', 'disable']
  pn_router_type:
    description:
      - Specify if the vRouter uses software or hardware.
      - Note that if you specify hardware as router type, you cannot assign IP
        addresses using DHCP. You must specify a static IP address.
    choices: ['hardware', 'software']
  pn_hw_vrrp_id:
    description:
      - Specifies the VRRP ID for a hardware vrouter.
  pn_router_id:
    description:
      - Specify the vRouter IP address.
  pn_bgp_as:
    description:
      - Specify the Autonomous System Number(ASN) if the vRouter runs Border
        Gateway Protocol(BGP).
  pn_bgp_redistribute:
    description:
      - Specify how BGP routes are redistributed.
    choices: ['static', 'connected', 'rip', 'ospf']
  pn_bgp_max_paths:
    description:
      - Specify the maximum number of paths for BGP. This is a number between
        1 and 255 or 0 to unset.
  pn_bgp_options:
    description:
      - Specify other BGP options as a whitespaces separated string within
        single quotes ''.
  pn_rip_redistribute:
    description:
      - Specify how RIP routes are redistributed.
    choices: ['static', 'connected', 'ospf', 'bgp']
  pn_ospf_redistribute:
    description:
      - Specify how OSPF routes are redistributed.
    choices: ['static', 'connected', 'bgp', 'rip']
  pn_ospf_options:
    description:
      - Specify other OSPF options as a whitespaces separated string within
        single quotes ''.
  pn_vrrp_track_port:
    description:
      - Specify list of ports and port ranges.
'''

EXAMPLES = """
- name: Create vrouter
  community.network.pn_vrouter:
    state: 'present'
    pn_name: 'ansible-vrouter'
    pn_vnet: 'ansible-fab-global'
    pn_router_id: 208.74.182.1

- name: Delete vrouter
  community.network.pn_vrouter:
    state: 'absent'
    pn_name: 'ansible-vrouter'
"""

RETURN = """
command:
  description: The CLI command run on the target node(s).
  returned: always
  type: str
stdout:
  description: The set of responses from the vrouter command.
  returned: always
  type: list
stderr:
  description: The set of error responses from the vrouter command.
  returned: on error
  type: list
changed:
  description: Indicates whether the CLI caused changes on the target.
  returned: always
  type: bool
"""

import shlex

# AnsibleModule boilerplate
from ansible.module_utils.basic import AnsibleModule

VROUTER_EXISTS = None
VROUTER_NAME_EXISTS = None


def pn_cli(module):
    """
    This method is to generate the cli portion to launch the Netvisor cli.
    It parses the username, password, switch parameters from module.
    :param module: The Ansible module to fetch username, password and switch
    :return: returns the cli string for further processing
    """
    username = module.params['pn_cliusername']
    password = module.params['pn_clipassword']
    cliswitch = module.params['pn_cliswitch']

    if username and password:
        cli = '/usr/bin/cli --quiet --user %s:%s ' % (username, password)
    else:
        cli = '/usr/bin/cli --quiet '

    if cliswitch == 'local':
        cli += ' switch-local '
    else:
        cli += ' switch ' + cliswitch
    return cli


def check_cli(module, cli):
    """
    This method checks for idempotency using the vlan-show command.
    A switch can have only one vRouter configuration.
    If a vRouter already exists on the given switch, return VROUTER_EXISTS as
    True else False.
    If a vRouter with the given name exists(on a different switch), return
    VROUTER_NAME_EXISTS as True else False.

    :param module: The Ansible module to fetch input parameters
    :param cli: The CLI string
    :return Global Booleans: VROUTER_EXISTS, VROUTER_NAME_EXISTS
    """
    name = module.params['pn_name']
    # Global flags
    global VROUTER_EXISTS, VROUTER_NAME_EXISTS

    # Get the name of the local switch
    location = cli + ' switch-setup-show format switch-name'
    location = shlex.split(location)
    out = module.run_command(location)[1]
    location = out.split()[1]

    # Check for any vRouters on the switch
    check_vrouter = cli + ' vrouter-show location %s ' % location
    check_vrouter += 'format name no-show-headers'
    check_vrouter = shlex.split(check_vrouter)
    out = module.run_command(check_vrouter)[1]

    if out:
        VROUTER_EXISTS = True
    else:
        VROUTER_EXISTS = False

    # Check for any vRouters with the given name
    show = cli + ' vrouter-show format name no-show-headers '
    show = shlex.split(show)
    out = module.run_command(show)[1]
    out = out.split()

    if name in out:
        VROUTER_NAME_EXISTS = True
    else:
        VROUTER_NAME_EXISTS = False


def run_cli(module, cli):
    """
    This method executes the cli command on the target node(s) and returns the
    output. The module then exits based on the output.
    :param cli: the complete cli string to be executed on the target node(s).
    :param module: The Ansible module to fetch command
    """
    cliswitch = module.params['pn_cliswitch']
    state = module.params['state']
    command = get_command_from_state(state)

    cmd = shlex.split(cli)

    # 'out' contains the output
    # 'err' contains the error messages
    result, out, err = module.run_command(cmd)

    print_cli = cli.split(cliswitch)[1]

    # Response in JSON format
    if result != 0:
        module.exit_json(
            command=print_cli,
            stderr=err.strip(),
            msg="%s operation failed" % command,
            changed=False
        )

    if out:
        module.exit_json(
            command=print_cli,
            stdout=out.strip(),
            msg="%s operation completed" % command,
            changed=True
        )

    else:
        module.exit_json(
            command=print_cli,
            msg="%s operation completed" % command,
            changed=True
        )


def get_command_from_state(state):
    """
    This method gets appropriate command name for the state specified. It
    returns the command name for the specified state.
    :param state: The state for which the respective command name is required.
    """
    command = None
    if state == 'present':
        command = 'vrouter-create'
    if state == 'absent':
        command = 'vrouter-delete'
    if state == 'update':
        command = 'vrouter-modify'
    return command


def main():
    """ This section is for arguments parsing """
    module = AnsibleModule(
        argument_spec=dict(
            pn_cliusername=dict(required=False, type='str'),
            pn_clipassword=dict(required=False, type='str', no_log=True),
            pn_cliswitch=dict(required=False, type='str', default='local'),
            state=dict(required=True, type='str',
                       choices=['present', 'absent', 'update']),
            pn_name=dict(required=True, type='str'),
            pn_vnet=dict(type='str'),
            pn_service_type=dict(type='str', choices=['dedicated', 'shared']),
            pn_service_state=dict(type='str', choices=['enable', 'disable']),
            pn_router_type=dict(type='str', choices=['hardware', 'software']),
            pn_hw_vrrp_id=dict(type='int'),
            pn_router_id=dict(type='str'),
            pn_bgp_as=dict(type='int'),
            pn_bgp_redistribute=dict(type='str', choices=['static', 'connected',
                                                          'rip', 'ospf']),
            pn_bgp_max_paths=dict(type='int'),
            pn_bgp_options=dict(type='str'),
            pn_rip_redistribute=dict(type='str', choices=['static', 'connected',
                                                          'bgp', 'ospf']),
            pn_ospf_redistribute=dict(type='str', choices=['static', 'connected',
                                                           'bgp', 'rip']),
            pn_ospf_options=dict(type='str'),
            pn_vrrp_track_port=dict(type='str')
        ),
        required_if=(
            ["state", "present", ["pn_name", "pn_vnet"]],
            ["state", "absent", ["pn_name"]],
            ["state", "update", ["pn_name"]]
        )
    )

    # Accessing the arguments
    state = module.params['state']
    name = module.params['pn_name']
    vnet = module.params['pn_vnet']
    service_type = module.params['pn_service_type']
    service_state = module.params['pn_service_state']
    router_type = module.params['pn_router_type']
    hw_vrrp_id = module.params['pn_hw_vrrp_id']
    router_id = module.params['pn_router_id']
    bgp_as = module.params['pn_bgp_as']
    bgp_redistribute = module.params['pn_bgp_redistribute']
    bgp_max_paths = module.params['pn_bgp_max_paths']
    bgp_options = module.params['pn_bgp_options']
    rip_redistribute = module.params['pn_rip_redistribute']
    ospf_redistribute = module.params['pn_ospf_redistribute']
    ospf_options = module.params['pn_ospf_options']
    vrrp_track_port = module.params['pn_vrrp_track_port']

    command = get_command_from_state(state)

    # Building the CLI command string
    cli = pn_cli(module)

    if command == 'vrouter-delete':
        check_cli(module, cli)
        if VROUTER_NAME_EXISTS is False:
            module.exit_json(
                skipped=True,
                msg='vRouter with name %s does not exist' % name
            )
        cli += ' %s name %s ' % (command, name)

    else:

        if command == 'vrouter-create':
            check_cli(module, cli)
            if VROUTER_EXISTS is True:
                module.exit_json(
                    skipped=True,
                    msg='Maximum number of vRouters has been reached on this '
                        'switch'
                )
            if VROUTER_NAME_EXISTS is True:
                module.exit_json(
                    skipped=True,
                    msg='vRouter with name %s already exists' % name
                )
        cli += ' %s name %s ' % (command, name)

        if vnet:
            cli += ' vnet ' + vnet

        if service_type:
            cli += ' %s-vnet-service ' % service_type

        if service_state:
            cli += ' ' + service_state

        if router_type:
            cli += ' router-type ' + router_type

        if hw_vrrp_id:
            cli += ' hw-vrrp-id ' + str(hw_vrrp_id)

        if router_id:
            cli += ' router-id ' + router_id

        if bgp_as:
            cli += ' bgp-as ' + str(bgp_as)

        if bgp_redistribute:
            cli += ' bgp-redistribute ' + bgp_redistribute

        if bgp_max_paths:
            cli += ' bgp-max-paths ' + str(bgp_max_paths)

        if bgp_options:
            cli += ' %s ' % bgp_options

        if rip_redistribute:
            cli += ' rip-redistribute ' + rip_redistribute

        if ospf_redistribute:
            cli += ' ospf-redistribute ' + ospf_redistribute

        if ospf_options:
            cli += ' %s ' % ospf_options

        if vrrp_track_port:
            cli += ' vrrp-track-port ' + vrrp_track_port

    run_cli(module, cli)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright: (c) 2018, Ansible Project
# Copyright: (c) 2018, Abhijeet Kasurde <akasurde@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: vmware_cluster_info
short_description: Gather info about clusters available in given vCenter
description:
    - This module can be used to gather information about clusters in VMWare infrastructure.
    - All values and VMware object names are case sensitive.
    - This module was called C(vmware_cluster_facts) before Ansible 2.9. The usage did not change.
author:
    - Abhijeet Kasurde (@Akasurde)
    - Christian Neugum (@digifuchsi)
notes:
    - Tested on vSphere 6.5, 6.7
requirements:
    - "python >= 2.6"
    - PyVmomi
options:
   datacenter:
     description:
     - Datacenter to search for cluster/s.
     - This parameter is required, if C(cluster_name) is not supplied.
     required: False
     type: str
   cluster_name:
     description:
     - Name of the cluster.
     - If set, information of this cluster will be returned.
     - This parameter is required, if C(datacenter) is not supplied.
     required: False
     type: str
   show_tag:
    description:
    - Tags related to cluster are shown if set to C(True).
    default: False
    type: bool
   schema:
     description:
       - Specify the output schema desired.
       - The 'summary' output schema is the legacy output from the module.
       - The 'vsphere' output schema is the vSphere API class definition which requires pyvmomi>6.7.1.
     choices: ['summary', 'vsphere']
     default: 'summary'
     type: str
     version_added: "1.0.0"
   properties:
     description:
       - Specify the properties to retrieve.
       - 'Example:'
       - '   properties: ['
       - '      "name",'
       - '      "configuration.dasConfig.enabled",'
       - '      "summary.totalCpu"'
       - '   ]'
       - Only valid when C(schema) is C(vsphere).
     type: list
     elements: str
     version_added: "1.0.0"
extends_documentation_fragment:
- community.vmware.vmware.documentation

'''

EXAMPLES = '''
- name: Gather cluster info from given datacenter
  community.vmware.vmware_cluster_info:
    hostname: '{{ vcenter_hostname }}'
    username: '{{ vcenter_username }}'
    password: '{{ vcenter_password }}'
    datacenter: ha-datacenter
    validate_certs: no
  delegate_to: localhost
  register: cluster_info

- name: Gather info from datacenter about specific cluster
  community.vmware.vmware_cluster_info:
    hostname: '{{ vcenter_hostname }}'
    username: '{{ vcenter_username }}'
    password: '{{ vcenter_password }}'
    cluster_name: DC0_C0
  delegate_to: localhost
  register: cluster_info

- name: Gather info from datacenter about specific cluster with tags
  community.vmware.vmware_cluster_info:
    hostname: '{{ vcenter_hostname }}'
    username: '{{ vcenter_username }}'
    password: '{{ vcenter_password }}'
    cluster_name: DC0_C0
    show_tag: True
  delegate_to: localhost
  register: cluster_info

- name: Gather some info from a cluster using the vSphere API output schema
  vmware_cluster_info:
    hostname: '{{ vcenter_hostname }}'
    username: '{{ vcenter_username }}'
    password: '{{ vcenter_password }}'
    validate_certs: no
    cluster_name: DC0_C0
    schema: vsphere
    properties:
      - name
      - configuration.dasConfig.enabled
      - summary.totalCpu
  delegate_to: localhost
  register: cluster_info
'''

RETURN = """
clusters:
    description: metadata about the available clusters
    returned: always
    type: dict
    sample: {
        "DC0_C0": {
            "drs_default_vm_behavior": null,
            "drs_enable_vm_behavior_overrides": null,
            "drs_vmotion_rate": null,
            "enable_ha": null,
            "enabled_drs": true,
            "enabled_vsan": false,
            "ha_admission_control_enabled": null,
            "ha_failover_level": null,
            "ha_host_monitoring": null,
            "ha_restart_priority": null,
            "ha_vm_failure_interval": null,
            "ha_vm_max_failure_window": null,
            "ha_vm_max_failures": null,
            "ha_vm_min_up_time": null,
            "ha_vm_monitoring": null,
            "ha_vm_tools_monitoring": null,
            "vsan_auto_claim_storage": false,
            "hosts": [
                {
                    "name": "esxi01.vsphere.local",
                    "folder": "/DC0/host/DC0_C0",
                },
                {
                    "name": "esxi02.vsphere.local",
                    "folder": "/DC0/host/DC0_C0",
                },
                {
                    "name": "esxi03.vsphere.local",
                    "folder": "/DC0/host/DC0_C0",
                },
                {
                    "name": "esxi04.vsphere.local",
                    "folder": "/DC0/host/DC0_C0",
                },
            ],
            "resource_summary": {
                "cpuCapacityMHz": 4224,
                "cpuUsedMHz": 87,
                "memCapacityMB": 6139,
                "memUsedMB": 1254,
                "pMemAvailableMB": 0,
                "pMemCapacityMB": 0,
                "storageCapacityMB": 33280,
                "storageUsedMB": 19953
            },
            "tags": [
                {
                    "category_id": "urn:vmomi:InventoryServiceCategory:9fbf83de-7903-442e-8004-70fd3940297c:GLOBAL",
                    "category_name": "sample_cluster_cat_0001",
                    "description": "",
                    "id": "urn:vmomi:InventoryServiceTag:93d680db-b3a6-4834-85ad-3e9516e8fee8:GLOBAL",
                    "name": "sample_cluster_tag_0001"
                }
            ],
        },
    }
"""

try:
    from pyVmomi import vim
except ImportError:
    pass

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.vmware.plugins.module_utils.vmware import PyVmomi, vmware_argument_spec, find_datacenter_by_name, find_cluster_by_name
from ansible_collections.community.vmware.plugins.module_utils.vmware_rest_client import VmwareRestClient


class VmwreClusterInfoManager(PyVmomi):
    def __init__(self, module):
        super(VmwreClusterInfoManager, self).__init__(module)
        datacenter = self.params.get('datacenter')
        cluster_name = self.params.get('cluster_name')
        self.schema = self.params.get('schema')
        self.properties = self.params.get('properties')
        self.cluster_objs = []
        if datacenter:
            datacenter_obj = find_datacenter_by_name(self.content, datacenter_name=datacenter)
            if datacenter_obj is None:
                self.module.fail_json(msg="Failed to find datacenter '%s'" % datacenter)
            self.cluster_objs = self.get_all_cluster_objs(parent=datacenter_obj)
        elif cluster_name:
            cluster_obj = find_cluster_by_name(self.content, cluster_name=cluster_name)
            if cluster_obj is None:
                self.module.fail_json(msg="Failed to find cluster '%s'" % cluster_name)

            self.cluster_objs = [cluster_obj]

    def get_all_cluster_objs(self, parent):
        """
        Get all cluster managed objects from given parent object
        Args:
            parent: Managed objected of datacenter or host folder

        Returns: List of host managed objects

        """
        cluster_objs = []
        if isinstance(parent, vim.Datacenter):
            folder = parent.hostFolder
        else:
            folder = parent

        for child in folder.childEntity:
            if isinstance(child, vim.Folder):
                cluster_objs = cluster_objs + self.get_all_cluster_objs(child)
            if isinstance(child, vim.ClusterComputeResource):
                cluster_objs.append(child)
        return cluster_objs

    def gather_cluster_info(self):
        """
        Gather information about cluster
        """
        results = dict(changed=False, clusters=dict())

        if self.schema == 'summary':
            for cluster in self.cluster_objs:
                # Default values
                ha_failover_level = None
                ha_restart_priority = None
                ha_vm_tools_monitoring = None
                ha_vm_min_up_time = None
                ha_vm_max_failures = None
                ha_vm_max_failure_window = None
                ha_vm_failure_interval = None
                enabled_vsan = False
                vsan_auto_claim_storage = False
                hosts = []

                # Hosts
                for host in cluster.host:
                    hosts.append({
                        'name': host.name,
                        'folder': self.get_vm_path(self.content, host),
                    })

                # HA
                das_config = cluster.configurationEx.dasConfig
                if das_config.admissionControlPolicy:
                    ha_failover_level = das_config.admissionControlPolicy.failoverLevel
                if das_config.defaultVmSettings:
                    ha_restart_priority = das_config.defaultVmSettings.restartPriority,
                    ha_vm_tools_monitoring = das_config.defaultVmSettings.vmToolsMonitoringSettings.vmMonitoring,
                    ha_vm_min_up_time = das_config.defaultVmSettings.vmToolsMonitoringSettings.minUpTime,
                    ha_vm_max_failures = das_config.defaultVmSettings.vmToolsMonitoringSettings.maxFailures,
                    ha_vm_max_failure_window = das_config.defaultVmSettings.vmToolsMonitoringSettings.maxFailureWindow,
                    ha_vm_failure_interval = das_config.defaultVmSettings.vmToolsMonitoringSettings.failureInterval,

                # DRS
                drs_config = cluster.configurationEx.drsConfig

                # VSAN
                if hasattr(cluster.configurationEx, 'vsanConfig'):
                    vsan_config = cluster.configurationEx.vsanConfig
                    enabled_vsan = vsan_config.enabled,
                    vsan_auto_claim_storage = vsan_config.defaultConfig.autoClaimStorage,

                tag_info = []
                if self.params.get('show_tag'):
                    vmware_client = VmwareRestClient(self.module)
                    tag_info = vmware_client.get_tags_for_cluster(cluster_mid=cluster._moId)

                resource_summary = self.to_json(cluster.GetResourceUsage())
                if '_vimtype' in resource_summary:
                    del resource_summary['_vimtype']

                results['clusters'][cluster.name] = dict(
                    hosts=hosts,
                    enable_ha=das_config.enabled,
                    ha_failover_level=ha_failover_level,
                    ha_vm_monitoring=das_config.vmMonitoring,
                    ha_host_monitoring=das_config.hostMonitoring,
                    ha_admission_control_enabled=das_config.admissionControlEnabled,
                    ha_restart_priority=ha_restart_priority,
                    ha_vm_tools_monitoring=ha_vm_tools_monitoring,
                    ha_vm_min_up_time=ha_vm_min_up_time,
                    ha_vm_max_failures=ha_vm_max_failures,
                    ha_vm_max_failure_window=ha_vm_max_failure_window,
                    ha_vm_failure_interval=ha_vm_failure_interval,
                    enabled_drs=drs_config.enabled,
                    drs_enable_vm_behavior_overrides=drs_config.enableVmBehaviorOverrides,
                    drs_default_vm_behavior=drs_config.defaultVmBehavior,
                    drs_vmotion_rate=drs_config.vmotionRate,
                    enabled_vsan=enabled_vsan,
                    vsan_auto_claim_storage=vsan_auto_claim_storage,
                    tags=tag_info,
                    resource_summary=resource_summary
                )
        else:
            for cluster in self.cluster_objs:
                results['clusters'][cluster.name] = self.to_json(cluster, self.properties)

        self.module.exit_json(**results)


def main():
    argument_spec = vmware_argument_spec()
    argument_spec.update(
        datacenter=dict(type='str'),
        cluster_name=dict(type='str'),
        show_tag=dict(type='bool', default=False),
        schema=dict(type='str', choices=['summary', 'vsphere'], default='summary'),
        properties=dict(type='list', elements='str')
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        required_one_of=[
            ['cluster_name', 'datacenter'],
        ],
        supports_check_mode=True,
    )
    if module._name == 'vmware_cluster_facts':
        module.deprecate(
            msg="The 'vmware_cluster_facts' module has been renamed to 'vmware_cluster_info'",
            version='3.0.0',  # was Ansible 2.13
            collection_name='community.vmware')

    pyv = VmwreClusterInfoManager(module)
    pyv.gather_cluster_info()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
---
module: ce_vxlan_arp
short_description: Manages ARP attributes of VXLAN on HUAWEI CloudEngine devices.
description:
    - Manages ARP attributes of VXLAN on HUAWEI CloudEngine devices.
author: QijunPan (@QijunPan)
notes:
    - Recommended connection is C(network_cli).
    - This module also works with C(local) connections for legacy playbooks.
options:
    evn_bgp:
        description:
            - Enables EVN BGP.
        choices: ['enable', 'disable']
    evn_source_ip:
        description:
            - Specifies the source address of an EVN BGP peer.
              The value is in dotted decimal notation.
    evn_peer_ip:
        description:
            - Specifies the IP address of an EVN BGP peer.
              The value is in dotted decimal notation.
    evn_server:
        description:
            - Configures the local device as the router reflector (RR) on the EVN network.
        choices: ['enable', 'disable']
    evn_reflect_client:
        description:
            - Configures the local device as the route reflector (RR) and its peer as the client.
        choices: ['enable', 'disable']
    vbdif_name:
        description:
            -  Full name of VBDIF interface, i.e. Vbdif100.
    arp_collect_host:
        description:
            - Enables EVN BGP or BGP EVPN to collect host information.
        choices: ['enable', 'disable']
    host_collect_protocol:
        description:
            - Enables EVN BGP or BGP EVPN to advertise host information.
        choices: ['bgp','none']
    bridge_domain_id:
        description:
            - Specifies a BD(bridge domain) ID.
              The value is an integer ranging from 1 to 16777215.
    arp_suppress:
        description:
            - Enables ARP broadcast suppression in a BD.
        choices: ['enable', 'disable']
    state:
        description:
            - Determines whether the config should be present or not
              on the device.
        default: present
        choices: ['present', 'absent']
'''

EXAMPLES = '''
- name: Vxlan arp module test
  hosts: ce128
  connection: local
  gather_facts: no
  vars:
    cli:
      host: "{{ inventory_hostname }}"
      port: "{{ ansible_ssh_port }}"
      username: "{{ username }}"
      password: "{{ password }}"
      transport: cli

  tasks:

  - name: Configure EVN BGP on Layer 2 and Layer 3 VXLAN gateways to establish EVN BGP peer relationships.
    community.network.ce_vxlan_arp:
      evn_bgp: enable
      evn_source_ip: 6.6.6.6
      evn_peer_ip: 7.7.7.7
      provider: "{{ cli }}"
  - name: Configure a Layer 3 VXLAN gateway as a BGP RR.
    community.network.ce_vxlan_arp:
      evn_bgp: enable
      evn_server: enable
      provider: "{{ cli }}"
  - name: Enable EVN BGP on a Layer 3 VXLAN gateway to collect host information.
    community.network.ce_vxlan_arp:
      vbdif_name: Vbdif100
      arp_collect_host: enable
      provider: "{{ cli }}"
  - name: Enable Layer 2 and Layer 3 VXLAN gateways to use EVN BGP to advertise host information.
    community.network.ce_vxlan_arp:
      host_collect_protocol: bgp
      provider: "{{ cli }}"
  - name: Enable ARP broadcast suppression on a Layer 2 VXLAN gateway.
    community.network.ce_vxlan_arp:
      bridge_domain_id: 100
      arp_suppress: enable
      provider: "{{ cli }}"
'''

RETURN = '''
proposed:
    description: k/v pairs of parameters passed into module
    returned: verbose mode
    type: dict
    sample: {"evn_bgp": "enable", "evn_source_ip": "6.6.6.6", "evn_peer_ip":"7.7.7.7", state: "present"}
existing:
    description: k/v pairs of existing configuration
    returned: verbose mode
    type: dict
    sample: {"evn_bgp": "disable", "evn_source_ip": null, "evn_peer_ip": []}
end_state:
    description: k/v pairs of configuration after module execution
    returned: verbose mode
    type: dict
    sample: {"evn_bgp": "enable", "evn_source_ip": "6.6.6.6", "evn_peer_ip": ["7.7.7.7"]}
updates:
    description: commands sent to the device
    returned: always
    type: list
    sample: ["evn bgp",
             "source-address 6.6.6.6",
             "peer 7.7.7.7"]
changed:
    description: check to see if a change was made on the device
    returned: always
    type: bool
    sample: true
'''

import re
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.network.plugins.module_utils.network.cloudengine.ce import load_config
from ansible_collections.community.network.plugins.module_utils.network.cloudengine.ce import ce_argument_spec
from ansible.module_utils.connection import exec_command


def is_config_exist(cmp_cfg, test_cfg):
    """is configuration exist"""

    if not cmp_cfg or not test_cfg:
        return False

    return bool(test_cfg in cmp_cfg)


def is_valid_v4addr(addr):
    """check is ipv4 addr is valid"""

    if addr.count('.') == 3:
        addr_list = addr.split('.')
        if len(addr_list) != 4:
            return False
        for each_num in addr_list:
            if not each_num.isdigit():
                return False
            if int(each_num) > 255:
                return False
        return True

    return False


def get_evn_peers(config):
    """get evn peer ip list"""

    get = re.findall(r"peer ([0-9]+.[0-9]+.[0-9]+.[0-9]+)", config)
    if not get:
        return None
    else:
        return list(set(get))


def get_evn_srouce(config):
    """get evn peer ip list"""

    get = re.findall(
        r"source-address ([0-9]+.[0-9]+.[0-9]+.[0-9]+)", config)
    if not get:
        return None
    else:
        return get[0]


def get_evn_reflect_client(config):
    """get evn reflect client list"""

    get = re.findall(
        r"peer ([0-9]+.[0-9]+.[0-9]+.[0-9]+)\s*reflect-client", config)
    if not get:
        return None
    else:
        return list(get)


class VxlanArp(object):
    """
    Manages arp attributes of VXLAN.
    """

    def __init__(self, argument_spec):
        self.spec = argument_spec
        self.module = None
        self.init_module()

        # module input info
        self.evn_bgp = self.module.params['evn_bgp']
        self.evn_source_ip = self.module.params['evn_source_ip']
        self.evn_peer_ip = self.module.params['evn_peer_ip']
        self.evn_server = self.module.params['evn_server']
        self.evn_reflect_client = self.module.params['evn_reflect_client']
        self.vbdif_name = self.module.params['vbdif_name']
        self.arp_collect_host = self.module.params['arp_collect_host']
        self.host_collect_protocol = self.module.params[
            'host_collect_protocol']
        self.bridge_domain_id = self.module.params['bridge_domain_id']
        self.arp_suppress = self.module.params['arp_suppress']
        self.state = self.module.params['state']

        # host info
        self.host = self.module.params['host']
        self.username = self.module.params['username']
        self.port = self.module.params['port']

        # state
        self.config = ""  # current config
        self.changed = False
        self.updates_cmd = list()
        self.commands = list()
        self.results = dict()
        self.proposed = dict()
        self.existing = dict()
        self.end_state = dict()

    def init_module(self):
        """init module"""

        required_together = [("vbdif_name", "arp_collect_host"), ("bridge_domain_id", "arp_suppress")]
        self.module = AnsibleModule(argument_spec=self.spec,
                                    required_together=required_together,
                                    supports_check_mode=True)

    def cli_load_config(self, commands):
        """load config by cli"""

        if not self.module.check_mode:
            load_config(self.module, commands)

    def get_config(self, flags=None):
        """Retrieves the current config from the device or cache
        """
        flags = [] if flags is None else flags

        cmd = 'display current-configuration '
        cmd += ' '.join(flags)
        cmd = cmd.strip()

        rc, out, err = exec_command(self.module, cmd)
        if rc != 0:
            self.module.fail_json(msg=err)
        cfg = str(out).strip()

        return cfg

    def get_current_config(self):
        """get current configuration"""

        flags = list()
        exp = r"| ignore-case section include evn bgp|host collect protocol bgp"
        if self.vbdif_name:
            exp += r"|^#\s+interface %s\s+" % self.vbdif_name.lower().capitalize().replace(" ", "")

        if self.bridge_domain_id:
            exp += r"|^#\s+bridge-domain %s\s+" % self.bridge_domain_id

        flags.append(exp)
        cfg_str = self.get_config(flags)
        config = cfg_str.split("\n")

        exist_config = ""
        for cfg in config:
            if not cfg.startswith("display"):
                exist_config += cfg
        return exist_config

    def cli_add_command(self, command, undo=False):
        """add command to self.update_cmd and self.commands"""

        if undo and command.lower() not in ["quit", "return"]:
            cmd = "undo " + command
        else:
            cmd = command

        self.commands.append(cmd)          # set to device
        if command.lower() not in ["quit", "return"]:
            self.updates_cmd.append(cmd)   # show updates result

    def config_bridge_domain(self):
        """manage bridge domain configuration"""

        if not self.bridge_domain_id:
            return

        # bridge-domain bd-id
        # [undo] arp broadcast-suppress enable

        cmd = "bridge-domain %s" % self.bridge_domain_id
        if not is_config_exist(self.config, cmd):
            self.module.fail_json(msg="Error: Bridge domain %s is not exist." % self.bridge_domain_id)

        cmd = "arp broadcast-suppress enable"
        exist = is_config_exist(self.config, cmd)
        if self.arp_suppress == "enable" and not exist:
            self.cli_add_command("bridge-domain %s" % self.bridge_domain_id)
            self.cli_add_command(cmd)
            self.cli_add_command("quit")
        elif self.arp_suppress == "disable" and exist:
            self.cli_add_command("bridge-domain %s" % self.bridge_domain_id)
            self.cli_add_command(cmd, undo=True)
            self.cli_add_command("quit")

    def config_evn_bgp(self):
        """enables EVN BGP and configure evn bgp command"""

        evn_bgp_view = False
        evn_bgp_enable = False

        cmd = "evn bgp"
        exist = is_config_exist(self.config, cmd)
        if self.evn_bgp == "enable" or exist:
            evn_bgp_enable = True

        # [undo] evn bgp
        if self.evn_bgp:
            if self.evn_bgp == "enable" and not exist:
                self.cli_add_command(cmd)
                evn_bgp_view = True
            elif self.evn_bgp == "disable" and exist:
                self.cli_add_command(cmd, undo=True)
                return

        # [undo] source-address ip-address
        if evn_bgp_enable and self.evn_source_ip:
            cmd = "source-address %s" % self.evn_source_ip
            exist = is_config_exist(self.config, cmd)
            if self.state == "present" and not exist:
                if not evn_bgp_view:
                    self.cli_add_command("evn bgp")
                    evn_bgp_view = True
                self.cli_add_command(cmd)
            elif self.state == "absent" and exist:
                if not evn_bgp_view:
                    self.cli_add_command("evn bgp")
                    evn_bgp_view = True
                self.cli_add_command(cmd, undo=True)

        # [undo] peer ip-address
        # [undo] peer ipv4-address reflect-client
        if evn_bgp_enable and self.evn_peer_ip:
            cmd = "peer %s" % self.evn_peer_ip
            exist = is_config_exist(self.config, cmd)
            if self.state == "present":
                if not exist:
                    if not evn_bgp_view:
                        self.cli_add_command("evn bgp")
                        evn_bgp_view = True
                    self.cli_add_command(cmd)
                    if self.evn_reflect_client == "enable":
                        self.cli_add_command(
                            "peer %s reflect-client" % self.evn_peer_ip)
                else:
                    if self.evn_reflect_client:
                        cmd = "peer %s reflect-client" % self.evn_peer_ip
                        exist = is_config_exist(self.config, cmd)
                        if self.evn_reflect_client == "enable" and not exist:
                            if not evn_bgp_view:
                                self.cli_add_command("evn bgp")
                                evn_bgp_view = True
                            self.cli_add_command(cmd)
                        elif self.evn_reflect_client == "disable" and exist:
                            if not evn_bgp_view:
                                self.cli_add_command("evn bgp")
                                evn_bgp_view = True
                            self.cli_add_command(cmd, undo=True)
            else:
                if exist:
                    if not evn_bgp_view:
                        self.cli_add_command("evn bgp")
                        evn_bgp_view = True
                    self.cli_add_command(cmd, undo=True)

        # [undo] server enable
        if evn_bgp_enable and self.evn_server:
            cmd = "server enable"
            exist = is_config_exist(self.config, cmd)
            if self.evn_server == "enable" and not exist:
                if not evn_bgp_view:
                    self.cli_add_command("evn bgp")
                    evn_bgp_view = True
                self.cli_add_command(cmd)
            elif self.evn_server == "disable" and exist:
                if not evn_bgp_view:
                    self.cli_add_command("evn bgp")
                    evn_bgp_view = True
                self.cli_add_command(cmd, undo=True)

        if evn_bgp_view:
            self.cli_add_command("quit")

    def config_vbdif(self):
        """configure command at the VBDIF interface view"""

        # interface vbdif bd-id
        # [undo] arp collect host enable

        cmd = "interface %s" % self.vbdif_name.lower().capitalize()
        exist = is_config_exist(self.config, cmd)

        if not exist:
            self.module.fail_json(
                msg="Error: Interface %s does not exist." % self.vbdif_name)

        cmd = "arp collect host enable"
        exist = is_config_exist(self.config, cmd)
        if self.arp_collect_host == "enable" and not exist:
            self.cli_add_command("interface %s" %
                                 self.vbdif_name.lower().capitalize())
            self.cli_add_command(cmd)
            self.cli_add_command("quit")
        elif self.arp_collect_host == "disable" and exist:
            self.cli_add_command("interface %s" %
                                 self.vbdif_name.lower().capitalize())
            self.cli_add_command(cmd, undo=True)
            self.cli_add_command("quit")

    def config_host_collect_protocal(self):
        """Enable EVN BGP or BGP EVPN to advertise host information"""

        # [undo] host collect protocol bgp
        cmd = "host collect protocol bgp"
        exist = is_config_exist(self.config, cmd)

        if self.state == "present":
            if self.host_collect_protocol == "bgp" and not exist:
                self.cli_add_command(cmd)
            elif self.host_collect_protocol == "none" and exist:
                self.cli_add_command(cmd, undo=True)
        else:
            if self.host_collect_protocol == "bgp" and exist:
                self.cli_add_command(cmd, undo=True)

    def is_valid_vbdif(self, ifname):
        """check is interface vbdif is valid"""

        if not ifname.upper().startswith('VBDIF'):
            return False
        bdid = self.vbdif_name.replace(" ", "").upper().replace("VBDIF", "")
        if not bdid.isdigit():
            return False
        if int(bdid) < 1 or int(bdid) > 16777215:
            return False

        return True

    def check_params(self):
        """Check all input params"""

        # bridge domain id check
        if self.bridge_domain_id:
            if not self.bridge_domain_id.isdigit():
                self.module.fail_json(
                    msg="Error: Bridge domain id is not digit.")
            if int(self.bridge_domain_id) < 1 or int(self.bridge_domain_id) > 16777215:
                self.module.fail_json(
                    msg="Error: Bridge domain id is not in the range from 1 to 16777215.")

        # evn_source_ip check
        if self.evn_source_ip:
            if not is_valid_v4addr(self.evn_source_ip):
                self.module.fail_json(msg="Error: evn_source_ip is invalid.")

        # evn_peer_ip check
        if self.evn_peer_ip:
            if not is_valid_v4addr(self.evn_peer_ip):
                self.module.fail_json(msg="Error: evn_peer_ip is invalid.")

        # vbdif_name check
        if self.vbdif_name:
            self.vbdif_name = self.vbdif_name.replace(
                " ", "").lower().capitalize()
            if not self.is_valid_vbdif(self.vbdif_name):
                self.module.fail_json(msg="Error: vbdif_name is invalid.")

        # evn_reflect_client and evn_peer_ip must set at the same time
        if self.evn_reflect_client and not self.evn_peer_ip:
            self.module.fail_json(
                msg="Error: evn_reflect_client and evn_peer_ip must set at the same time.")

        # evn_server and evn_reflect_client can not set at the same time
        if self.evn_server == "enable" and self.evn_reflect_client == "enable":
            self.module.fail_json(
                msg="Error: evn_server and evn_reflect_client can not set at the same time.")

    def get_proposed(self):
        """get proposed info"""

        if self.evn_bgp:
            self.proposed["evn_bgp"] = self.evn_bgp
        if self.evn_source_ip:
            self.proposed["evn_source_ip"] = self.evn_source_ip
        if self.evn_peer_ip:
            self.proposed["evn_peer_ip"] = self.evn_peer_ip
        if self.evn_server:
            self.proposed["evn_server"] = self.evn_server
        if self.evn_reflect_client:
            self.proposed["evn_reflect_client"] = self.evn_reflect_client
        if self.arp_collect_host:
            self.proposed["arp_collect_host"] = self.arp_collect_host
        if self.host_collect_protocol:
            self.proposed["host_collect_protocol"] = self.host_collect_protocol
        if self.arp_suppress:
            self.proposed["arp_suppress"] = self.arp_suppress
        if self.vbdif_name:
            self.proposed["vbdif_name"] = self.evn_peer_ip
        if self.bridge_domain_id:
            self.proposed["bridge_domain_id"] = self.bridge_domain_id
        self.proposed["state"] = self.state

    def get_existing(self):
        """get existing info"""

        evn_bgp_exist = is_config_exist(self.config, "evn bgp")
        if evn_bgp_exist:
            self.existing["evn_bgp"] = "enable"
        else:
            self.existing["evn_bgp"] = "disable"

        if evn_bgp_exist:
            if is_config_exist(self.config, "server enable"):
                self.existing["evn_server"] = "enable"
            else:
                self.existing["evn_server"] = "disable"

            self.existing["evn_source_ip"] = get_evn_srouce(self.config)
            self.existing["evn_peer_ip"] = get_evn_peers(self.config)
            self.existing["evn_reflect_client"] = get_evn_reflect_client(
                self.config)

        if is_config_exist(self.config, "arp collect host enable"):
            self.existing["host_collect_protocol"] = "enable"
        else:
            self.existing["host_collect_protocol"] = "disable"

        if is_config_exist(self.config, "host collect protocol bgp"):
            self.existing["host_collect_protocol"] = "bgp"
        else:
            self.existing["host_collect_protocol"] = None

        if is_config_exist(self.config, "arp broadcast-suppress enable"):
            self.existing["arp_suppress"] = "enable"
        else:
            self.existing["arp_suppress"] = "disable"

    def get_end_state(self):
        """get end state info"""

        config = self.get_current_config()
        evn_bgp_exist = is_config_exist(config, "evn bgp")
        if evn_bgp_exist:
            self.end_state["evn_bgp"] = "enable"
        else:
            self.end_state["evn_bgp"] = "disable"

        if evn_bgp_exist:
            if is_config_exist(config, "server enable"):
                self.end_state["evn_server"] = "enable"
            else:
                self.end_state["evn_server"] = "disable"

            self.end_state["evn_source_ip"] = get_evn_srouce(config)
            self.end_state["evn_peer_ip"] = get_evn_peers(config)
            self.end_state[
                "evn_reflect_client"] = get_evn_reflect_client(config)

        if is_config_exist(config, "arp collect host enable"):
            self.end_state["host_collect_protocol"] = "enable"
        else:
            self.end_state["host_collect_protocol"] = "disable"

        if is_config_exist(config, "host collect protocol bgp"):
            self.end_state["host_collect_protocol"] = "bgp"
        else:
            self.end_state["host_collect_protocol"] = None

        if is_config_exist(config, "arp broadcast-suppress enable"):
            self.end_state["arp_suppress"] = "enable"
        else:
            self.end_state["arp_suppress"] = "disable"

    def work(self):
        """worker"""

        self.check_params()
        self.config = self.get_current_config()
        self.get_existing()
        self.get_proposed()

        # deal present or absent
        if self.evn_bgp or self.evn_server or self.evn_peer_ip or self.evn_source_ip:
            self.config_evn_bgp()

        if self.vbdif_name and self.arp_collect_host:
            self.config_vbdif()

        if self.host_collect_protocol:
            self.config_host_collect_protocal()

        if self.bridge_domain_id and self.arp_suppress:
            self.config_bridge_domain()

        if self.commands:
            self.cli_load_config(self.commands)
            self.changed = True

        self.get_end_state()
        self.results['changed'] = self.changed
        self.results['proposed'] = self.proposed
        self.results['existing'] = self.existing
        self.results['end_state'] = self.end_state
        if self.changed:
            self.results['updates'] = self.updates_cmd
        else:
            self.results['updates'] = list()

        self.module.exit_json(**self.results)


def main():
    """Module main"""

    argument_spec = dict(
        evn_bgp=dict(required=False, type='str',
                     choices=['enable', 'disable']),
        evn_source_ip=dict(required=False, type='str'),
        evn_peer_ip=dict(required=False, type='str'),
        evn_server=dict(required=False, type='str',
                        choices=['enable', 'disable']),
        evn_reflect_client=dict(
            required=False, type='str', choices=['enable', 'disable']),
        vbdif_name=dict(required=False, type='str'),
        arp_collect_host=dict(required=False, type='str',
                              choices=['enable', 'disable']),
        host_collect_protocol=dict(
            required=False, type='str', choices=['bgp', 'none']),
        bridge_domain_id=dict(required=False, type='str'),
        arp_suppress=dict(required=False, type='str',
                          choices=['enable', 'disable']),
        state=dict(required=False, default='present',
                   choices=['present', 'absent'])
    )
    argument_spec.update(ce_argument_spec)
    module = VxlanArp(argument_spec)
    module.work()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = '''
---
module: azure_rm_image_info

version_added: "2.9"

short_description: Get facts about azure custom images

description:
    - List azure custom images. The images can be listed where scope of listing can be based on subscription, resource group, name or tags.

options:
    resource_group:
        description:
            - Name of resource group.
    name:
        description:
            - Name of the image to filter from existing images.
    tags:
        description:
            - List of tags to be matched.

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Madhura Naniwadekar (@Madhura-CSI)
'''


EXAMPLES = '''
- name: List images with name
  azure_rm_image_info:
    name: test-image
    resource_group: myResourceGroup

- name: List images by resource group
  azure_rm_image_info:
    resource_group: myResourceGroup
    tags:
      - testing
      - foo:bar

- name: List all available images under current subscription
  azure_rm_image_info:
'''


RETURN = '''
images:
    description:
        - List of image dicts.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Id of the image.
            returned: always
            type: str
            sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Compute/images/xx
        name:
            description:
                - Name of the image.
            returned: always
            type: str
        resource_group:
            description:
                - Resource group of the image.
            returned: always
            type: str
            sample: myResourceGroup
        location:
            description:
                - Location of the image.
            returned: always
            type: str
        os_disk:
            description:
                - Id of os disk for image.
            type: str
            sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Compute/disks/xx
        os_disk_caching:
            description:
                - Specifies caching requirements for the image.
            returned: always
            type: str
        os_state:
            description:
                - Specifies image operating system state. Possible values are C(Generalized) or C(Specialized).
            returned: always
            type: str
            sample: Generalized
        os_storage_account_type:
            description:
                - Specifies the storage account type for the managed disk.
            type: str
            sample: Standard_LRS
        os_type:
            description:
                - Type of OS for image.
            returned: always
            type: str
            sample: Linux
        provisioning_state:
            description:
                - State of image.
            returned: always
            type: str
            sample: Succeeded
        source:
            description:
                - Resource id of source VM from which the image is created.
            type: str
            sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Compute/virtualMachines/xx
        tags:
            description:
                - Dictionary of tags associated with the image.
            type: complex
        data_disks:
            description:
                - List of data disks associated with the image.
            type: complex
            returned: always
            contains:
                caching:
                    description:
                        - Type of caching of data disk.
                    sample: read_only
                disk_size_gb:
                    description:
                        - Specifies the size of empty data disks in gigabytes.
                    returned: always
                    type: int
                    sample: 50
                lun:
                    description:
                        - Specifies the logical unit number of the data disk.
                    returned: always
                    type: int
                    sample: 0
                storage_account_type:
                    description:
                        - Specifies the storage account type for the managed disk data disk.
                    type: str
                    sample: Standard_LRS
                managed_disk_id:
                    description:
                        - Id of managed disk.
                    type: str
                    sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Compute/disks/xx
                blob_uri:
                    description:
                        - The virtual hard disk.
'''


try:
    from msrestazure.azure_exceptions import CloudError
except Exception:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


AZURE_ENUM_MODULES = ['azure.mgmt.compute.models']


class AzureRMImageInfo(AzureRMModuleBase):

    def __init__(self, **kwargs):

        self.module_arg_spec = dict(
            resource_group=dict(type='str'),
            name=dict(type='str'),
            tags=dict(type='list')
        )

        self.results = dict(
            changed=False
        )

        self.resource_group = None
        self.name = None
        self.format = None
        self.tags = None

        super(AzureRMImageInfo, self).__init__(
            derived_arg_spec=self.module_arg_spec,
            supports_tags=False,
            facts_module=True
        )

    def exec_module(self, **kwargs):

        is_old_facts = self.module._name == 'azure_rm_image_facts'
        if is_old_facts:
            self.module.deprecate("The 'azure_rm_image_facts' module has been renamed to 'azure_rm_image_info'", version=(2.9, ))

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name and self.resource_group:
            self.results['images'] = self.get_image(self.resource_group, self.name)
        elif self.name and not self.resource_group:
            self.results['images'] = self.list_images(self.name)
        elif not self.name and self.resource_group:
            self.results['images'] = self.list_images_by_resource_group(self.resource_group)
        elif not self.name and not self.resource_group:
            self.results['images'] = self.list_images()
        return self.results

    def get_image(self, resource_group, image_name):
        '''
        Returns image details based on its name
        '''

        self.log('Get properties for {0}'.format(self.name))

        result = []
        item = None
        try:
            item = self.compute_client.images.get(resource_group, image_name)
        except CloudError as exc:
            self.fail('Failed to list images - {0}'.format(str(exc)))

        result = [self.format_item(item)]
        return result

    def list_images_by_resource_group(self, resource_group):
        '''
        Returns image details based on its resource group
        '''

        self.log('List images filtered by resource group')
        response = None
        try:
            response = self.compute_client.images.list_by_resource_group(resource_group)
        except CloudError as exc:
            self.fail("Failed to list images: {0}".format(str(exc)))

        return [self.format_item(x) for x in response if self.has_tags(x.tags, self.tags)] if response else []

    def list_images(self, image_name=None):
        '''
        Returns image details in current subscription
        '''

        self.log('List images within current subscription')
        response = None
        results = []
        try:
            response = self.compute_client.images.list()
        except CloudError as exc:
            self.fail("Failed to list all images: {0}".format(str(exc)))

        results = [self.format_item(x) for x in response if self.has_tags(x.tags, self.tags)] if response else []
        if image_name:
            results = [result for result in results if result['name'] == image_name]
        return results

    def format_item(self, item):
        d = item.as_dict()

        for data_disk in d['storage_profile']['data_disks']:
            if 'managed_disk' in data_disk.keys():
                data_disk['managed_disk_id'] = data_disk['managed_disk']['id']
                data_disk.pop('managed_disk', None)

        d = {
            'id': d['id'],
            'resource_group': d['id'].split('/')[4],
            'name': d['name'],
            'location': d['location'],
            'tags': d.get('tags'),
            'source': d['source_virtual_machine']['id'] if 'source_virtual_machine' in d.keys() else None,
            'os_type': d['storage_profile']['os_disk']['os_type'],
            'os_state': d['storage_profile']['os_disk']['os_state'],
            'os_disk_caching': d['storage_profile']['os_disk']['caching'],
            'os_storage_account_type': d['storage_profile']['os_disk']['storage_account_type'],
            'os_disk': d['storage_profile']['os_disk']['managed_disk']['id'] if 'managed_disk' in d['storage_profile']['os_disk'].keys() else None,
            'os_blob_uri': d['storage_profile']['os_disk']['blob_uri'] if 'blob_uri' in d['storage_profile']['os_disk'].keys() else None,
            'provisioning_state': d['provisioning_state'],
            'data_disks': d['storage_profile']['data_disks']
        }
        return d


def main():
    AzureRMImageInfo()


if __name__ == '__main__':
    main()



# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
    name: virtualbox
    plugin_type: inventory
    short_description: virtualbox inventory source
    description:
        - Get inventory hosts from the local virtualbox installation.
        - Uses a YAML configuration file that ends with virtualbox.(yml|yaml) or vbox.(yml|yaml).
        - The inventory_hostname is always the 'Name' of the virtualbox instance.
    extends_documentation_fragment:
      - constructed
      - inventory_cache
    options:
        plugin:
            description: token that ensures this is a source file for the 'virtualbox' plugin
            required: True
            choices: ['virtualbox']
        running_only:
            description: toggles showing all vms vs only those currently running
            type: boolean
            default: False
        settings_password_file:
            description: provide a file containing the settings password (equivalent to --settingspwfile)
        network_info_path:
            description: property path to query for network information (ansible_host)
            default: "/VirtualBox/GuestInfo/Net/0/V4/IP"
        query:
            description: create vars from virtualbox properties
            type: dictionary
            default: {}
'''

EXAMPLES = '''
# file must be named vbox.yaml or vbox.yml
simple_config_file:
    plugin: virtualbox
    settings_password_file: /etc/virtulbox/secrets
    query:
      logged_in_users: /VirtualBox/GuestInfo/OS/LoggedInUsersList
    compose:
      ansible_connection: ('indows' in vbox_Guest_OS)|ternary('winrm', 'ssh')

# add hosts (all match with minishift vm) to the group container if any of the vms are in ansible_inventory'
plugin: virtualbox
groups:
  container: "'minis' in (inventory_hostname)"
'''

import os

from subprocess import Popen, PIPE

from ansible.errors import AnsibleParserError
from ansible.module_utils._text import to_bytes, to_native, to_text
from ansible.module_utils.common._collections_compat import MutableMapping
from ansible.plugins.inventory import BaseInventoryPlugin, Constructable, Cacheable
from ansible.module_utils.common.process import get_bin_path


class InventoryModule(BaseInventoryPlugin, Constructable, Cacheable):
    ''' Host inventory parser for ansible using local virtualbox. '''

    NAME = 'community.general.virtualbox'
    VBOX = "VBoxManage"

    def __init__(self):
        self._vbox_path = None
        super(InventoryModule, self).__init__()

    def _query_vbox_data(self, host, property_path):
        ret = None
        try:
            cmd = [self._vbox_path, b'guestproperty', b'get',
                   to_bytes(host, errors='surrogate_or_strict'),
                   to_bytes(property_path, errors='surrogate_or_strict')]
            x = Popen(cmd, stdout=PIPE)
            ipinfo = to_text(x.stdout.read(), errors='surrogate_or_strict')
            if 'Value' in ipinfo:
                a, ip = ipinfo.split(':', 1)
                ret = ip.strip()
        except Exception:
            pass
        return ret

    def _set_variables(self, hostvars):

        # set vars in inventory from hostvars
        for host in hostvars:

            query = self.get_option('query')
            # create vars from vbox properties
            if query and isinstance(query, MutableMapping):
                for varname in query:
                    hostvars[host][varname] = self._query_vbox_data(host, query[varname])

            strict = self.get_option('strict')

            # create composite vars
            self._set_composite_vars(self.get_option('compose'), hostvars[host], host, strict=strict)

            # actually update inventory
            for key in hostvars[host]:
                self.inventory.set_variable(host, key, hostvars[host][key])

            # constructed groups based on conditionals
            self._add_host_to_composed_groups(self.get_option('groups'), hostvars[host], host, strict=strict)

            # constructed keyed_groups
            self._add_host_to_keyed_groups(self.get_option('keyed_groups'), hostvars[host], host, strict=strict)

    def _populate_from_cache(self, source_data):
        hostvars = source_data.pop('_meta', {}).get('hostvars', {})
        for group in source_data:
            if group == 'all':
                continue
            else:
                group = self.inventory.add_group(group)
                hosts = source_data[group].get('hosts', [])
                for host in hosts:
                    self._populate_host_vars([host], hostvars.get(host, {}), group)
                self.inventory.add_child('all', group)
        if not source_data:
            for host in hostvars:
                self.inventory.add_host(host)
                self._populate_host_vars([host], hostvars.get(host, {}))

    def _populate_from_source(self, source_data, using_current_cache=False):
        if using_current_cache:
            self._populate_from_cache(source_data)
            return source_data

        cacheable_results = {'_meta': {'hostvars': {}}}

        hostvars = {}
        prevkey = pref_k = ''
        current_host = None

        # needed to possibly set ansible_host
        netinfo = self.get_option('network_info_path')

        for line in source_data:
            line = to_text(line)
            if ':' not in line:
                continue
            try:
                k, v = line.split(':', 1)
            except Exception:
                # skip non splitable
                continue

            if k.strip() == '':
                # skip empty
                continue

            v = v.strip()
            # found host
            if k.startswith('Name') and ',' not in v:  # some setting strings appear in Name
                current_host = v
                if current_host not in hostvars:
                    hostvars[current_host] = {}
                    self.inventory.add_host(current_host)

                # try to get network info
                netdata = self._query_vbox_data(current_host, netinfo)
                if netdata:
                    self.inventory.set_variable(current_host, 'ansible_host', netdata)

            # found groups
            elif k == 'Groups':
                for group in v.split('/'):
                    if group:
                        group = self.inventory.add_group(group)
                        self.inventory.add_child(group, current_host)
                        if group not in cacheable_results:
                            cacheable_results[group] = {'hosts': []}
                        cacheable_results[group]['hosts'].append(current_host)
                continue

            else:
                # found vars, accumulate in hostvars for clean inventory set
                pref_k = 'vbox_' + k.strip().replace(' ', '_')
                if k.startswith(' '):
                    if prevkey not in hostvars[current_host]:
                        hostvars[current_host][prevkey] = {}
                    hostvars[current_host][prevkey][pref_k] = v
                else:
                    if v != '':
                        hostvars[current_host][pref_k] = v
                if self._ungrouped_host(current_host, cacheable_results):
                    if 'ungrouped' not in cacheable_results:
                        cacheable_results['ungrouped'] = {'hosts': []}
                    cacheable_results['ungrouped']['hosts'].append(current_host)

                prevkey = pref_k

        self._set_variables(hostvars)
        for host in hostvars:
            h = self.inventory.get_host(host)
            cacheable_results['_meta']['hostvars'][h.name] = h.vars

        return cacheable_results

    def _ungrouped_host(self, host, inventory):
        def find_host(host, inventory):
            for k, v in inventory.items():
                if k == '_meta':
                    continue
                if isinstance(v, dict):
                    yield self._ungrouped_host(host, v)
                elif isinstance(v, list):
                    yield host not in v
            yield True

        return all([found_host for found_host in find_host(host, inventory)])

    def verify_file(self, path):

        valid = False
        if super(InventoryModule, self).verify_file(path):
            if path.endswith(('virtualbox.yaml', 'virtualbox.yml', 'vbox.yaml', 'vbox.yml')):
                valid = True
        return valid

    def parse(self, inventory, loader, path, cache=True):

        try:
            self._vbox_path = get_bin_path(self.VBOX)
        except ValueError as e:
            raise AnsibleParserError(e)

        super(InventoryModule, self).parse(inventory, loader, path)

        cache_key = self.get_cache_key(path)

        config_data = self._read_config_data(path)

        # set _options from config data
        self._consume_options(config_data)

        source_data = None
        if cache:
            cache = self.get_option('cache')

        update_cache = False
        if cache:
            try:
                source_data = self._cache[cache_key]
            except KeyError:
                update_cache = True

        if not source_data:
            b_pwfile = to_bytes(self.get_option('settings_password_file'), errors='surrogate_or_strict', nonstring='passthru')
            running = self.get_option('running_only')

            # start getting data
            cmd = [self._vbox_path, b'list', b'-l']
            if running:
                cmd.append(b'runningvms')
            else:
                cmd.append(b'vms')

            if b_pwfile and os.path.exists(b_pwfile):
                cmd.append(b'--settingspwfile')
                cmd.append(b_pwfile)

            try:
                p = Popen(cmd, stdout=PIPE)
            except Exception as e:
                raise AnsibleParserError(to_native(e))

            source_data = p.stdout.read().splitlines()

        using_current_cache = cache and not update_cache
        cacheable_results = self._populate_from_source(source_data, using_current_cache)

        if update_cache:
            self._cache[cache_key] = cacheable_results



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2017, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = """
module: nxos_l2_interface
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: (deprecated, removed after 2022-06-01) Manage Layer-2
  interface on Cisco NXOS devices.
description:
- This module provides declarative management of Layer-2 interface on Cisco NXOS devices.
version_added: 1.0.0
deprecated:
  alternative: nxos_l2_interfaces
  why: Updated modules released with more functionality
  removed_at_date: '2022-06-01'
author:
- Trishna Guha (@trishnaguha)
notes:
- Tested against NXOSv 7.0(3)I5(1).
options:
  name:
    description:
    - Full name of the interface excluding any logical unit number, i.e. Ethernet1/1.
    aliases:
    - interface
    type: str
  mode:
    description:
    - Mode in which interface needs to be configured.
    choices:
    - access
    - trunk
    type: str
  access_vlan:
    description:
    - Configure given VLAN in access port. If C(mode=access), used as the access VLAN
      ID.
    type: str
  native_vlan:
    description:
    - Native VLAN to be configured in trunk port. If C(mode=trunk), used as the trunk
      native VLAN ID.
    type: str
  trunk_vlans:
    description:
    - List of VLANs to be configured in trunk port. If C(mode=trunk), used as the
      VLAN range to ADD or REMOVE from the trunk.
    aliases:
    - trunk_add_vlans
    type: str
  trunk_allowed_vlans:
    description:
    - List of allowed VLANs in a given trunk port. If C(mode=trunk), these are the
      only VLANs that will be configured on the trunk, i.e. "2-10,15".
    type: str
  aggregate:
    description:
    - List of Layer-2 interface definitions.
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - Full name of the interface excluding any logical unit number, i.e. Ethernet1/1.
        aliases:
        - interface
        type: str
      mode:
        description:
        - Mode in which interface needs to be configured.
        choices:
        - access
        - trunk
        type: str
      access_vlan:
        description:
        - Configure given VLAN in access port. If C(mode=access), used as the access VLAN
          ID.
        type: str
      native_vlan:
        description:
        - Native VLAN to be configured in trunk port. If C(mode=trunk), used as the trunk
          native VLAN ID.
        type: str
      trunk_vlans:
        description:
        - List of VLANs to be configured in trunk port. If C(mode=trunk), used as the
          VLAN range to ADD or REMOVE from the trunk.
        aliases:
        - trunk_add_vlans
        type: str
      trunk_allowed_vlans:
        description:
        - List of allowed VLANs in a given trunk port. If C(mode=trunk), these are the
          only VLANs that will be configured on the trunk, i.e. "2-10,15".
        type: str
      state:
        description:
        - Manage the state of the Layer-2 Interface configuration.
        choices:
        - present
        - absent
        - unconfigured
        type: str

  state:
    description:
    - Manage the state of the Layer-2 Interface configuration.
    default: present
    choices:
    - present
    - absent
    - unconfigured
    type: str


"""

EXAMPLES = """
- name: Ensure Eth1/5 is in its default l2 interface state
  cisco.nxos.nxos_l2_interface:
    name: Ethernet1/5
    state: unconfigured

- name: Ensure Eth1/5 is configured for access vlan 20
  cisco.nxos.nxos_l2_interface:
    name: Ethernet1/5
    mode: access
    access_vlan: 20

- name: Ensure Eth1/5 only has vlans 5-10 as trunk vlans
  cisco.nxos.nxos_l2_interface:
    name: Ethernet1/5
    mode: trunk
    native_vlan: 10
    trunk_vlans: 5-10

- name: Ensure eth1/5 is a trunk port and ensure 2-50 are being tagged (doesn't mean
    others aren't also being tagged)
  cisco.nxos.nxos_l2_interface:
    name: Ethernet1/5
    mode: trunk
    native_vlan: 10
    trunk_vlans: 2-50

- name: Ensure these VLANs are not being tagged on the trunk
  cisco.nxos.nxos_l2_interface:
    name: Ethernet1/5
    mode: trunk
    trunk_vlans: 51-4094
    state: absent

- name: Aggregate Configure interfaces for access_vlan with aggregate
  cisco.nxos.nxos_l2_interface:
    aggregate:
    - {name: Ethernet1/2, access_vlan: 6}
    - {name: Ethernet1/7, access_vlan: 15}
    mode: access
"""

RETURN = """
commands:
  description: The list of configuration mode commands to send to the device
  returned: always, except for the platforms that use Netconf transport to manage the device.
  type: list
  sample:
    - interface eth1/5
    - switchport access vlan 20
"""

from copy import deepcopy

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    load_config,
    run_commands,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    nxos_argument_spec,
    get_interface_type,
)
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    remove_default_spec,
)


def get_interface_mode(name, module):
    """Gets current mode of interface: layer2 or layer3
    Args:
        device (Device): This is the device object of an NX-API enabled device
            using the Device class within device.py
        interface (string): full name of interface, i.e. Ethernet1/1,
            loopback10, port-channel20, vlan20
    Returns:
        str: 'layer2' or 'layer3'
    """
    command = "show interface {0} | json".format(name)
    intf_type = get_interface_type(name)
    mode = "unknown"
    interface_table = {}

    try:
        body = run_commands(module, [command])[0]
        interface_table = body["TABLE_interface"]["ROW_interface"]
    except (KeyError, AttributeError, IndexError):
        return mode

    if interface_table:
        # HACK FOR NOW
        if intf_type in ["ethernet", "portchannel"]:
            mode = str(interface_table.get("eth_mode", "layer3"))
            if mode in ["access", "trunk"]:
                mode = "layer2"
            if mode == "routed":
                mode = "layer3"
        elif intf_type == "loopback" or intf_type == "svi":
            mode = "layer3"
    return mode


def interface_is_portchannel(name, module):
    """Checks to see if an interface is part of portchannel bundle
    Args:
        interface (str): full name of interface, i.e. Ethernet1/1
    Returns:
        True/False based on if interface is a member of a portchannel bundle
    """
    intf_type = get_interface_type(name)

    if intf_type == "ethernet":
        command = "show interface {0} | json".format(name)
        try:
            body = run_commands(module, [command])[0]
            interface_table = body["TABLE_interface"]["ROW_interface"]
        except (KeyError, AttributeError, IndexError):
            interface_table = None

        if interface_table:
            state = interface_table.get("eth_bundle")
            if state:
                return True
            else:
                return False

    return False


def get_switchport(port, module):
    """Gets current config of L2 switchport
    Args:
        device (Device): This is the device object of an NX-API enabled device
            using the Device class within device.py
        port (str): full name of interface, i.e. Ethernet1/1
    Returns:
        dictionary with k/v pairs for L2 vlan config
    """

    command = "show interface {0} switchport | json".format(port)

    try:
        body = run_commands(module, [command])[0]
        sp_table = body["TABLE_interface"]["ROW_interface"]
    except (KeyError, AttributeError, IndexError):
        sp_table = None

    if sp_table:
        key_map = {
            "interface": "name",
            "oper_mode": "mode",
            "switchport": "switchport",
            "access_vlan": "access_vlan",
            "access_vlan_name": "access_vlan_name",
            "native_vlan": "native_vlan",
            "native_vlan_name": "native_vlan_name",
            "trunk_vlans": "trunk_vlans",
        }
        sp = apply_key_map(key_map, sp_table)
        return sp

    else:
        return {}


def remove_switchport_config_commands(name, existing, proposed, module):
    mode = proposed.get("mode")
    commands = []
    command = None

    if mode == "access":
        av_check = existing.get("access_vlan") == proposed.get("access_vlan")
        if av_check:
            command = "no switchport access vlan {0}".format(
                existing.get("access_vlan")
            )
            commands.append(command)

    elif mode == "trunk":
        existing_vlans = existing.get("trunk_vlans_list")
        proposed_vlans = proposed.get("trunk_vlans_list")
        vlans_to_remove = set(proposed_vlans).intersection(existing_vlans)

        if vlans_to_remove:
            proposed_allowed_vlans = proposed.get("trunk_allowed_vlans")
            remove_trunk_allowed_vlans = proposed.get(
                "trunk_vlans", proposed_allowed_vlans
            )
            command = "switchport trunk allowed vlan remove {0}".format(
                remove_trunk_allowed_vlans
            )
            commands.append(command)

        native_check = existing.get("native_vlan") == proposed.get(
            "native_vlan"
        )
        if native_check and proposed.get("native_vlan"):
            command = "no switchport trunk native vlan {0}".format(
                existing.get("native_vlan")
            )
            commands.append(command)

    if commands:
        commands.insert(0, "interface " + name)
    return commands


def get_switchport_config_commands(name, existing, proposed, module):
    """Gets commands required to config a given switchport interface
    """

    proposed_mode = proposed.get("mode")
    existing_mode = existing.get("mode")
    commands = []
    command = None

    if proposed_mode != existing_mode:
        if proposed_mode == "trunk":
            command = "switchport mode trunk"
        elif proposed_mode == "access":
            command = "switchport mode access"

    if command:
        commands.append(command)

    if proposed_mode == "access":
        av_check = str(existing.get("access_vlan")) == str(
            proposed.get("access_vlan")
        )
        if not av_check:
            command = "switchport access vlan {0}".format(
                proposed.get("access_vlan")
            )
            commands.append(command)

    elif proposed_mode == "trunk":
        tv_check = existing.get("trunk_vlans_list") == proposed.get(
            "trunk_vlans_list"
        )

        if not tv_check:
            if proposed.get("allowed"):
                command = "switchport trunk allowed vlan {0}".format(
                    proposed.get("trunk_allowed_vlans")
                )
                commands.append(command)

            else:
                existing_vlans = existing.get("trunk_vlans_list")
                proposed_vlans = proposed.get("trunk_vlans_list")
                vlans_to_add = set(proposed_vlans).difference(existing_vlans)
                if vlans_to_add:
                    command = "switchport trunk allowed vlan add {0}".format(
                        proposed.get("trunk_vlans")
                    )
                    commands.append(command)

        native_check = str(existing.get("native_vlan")) == str(
            proposed.get("native_vlan")
        )
        if not native_check and proposed.get("native_vlan"):
            command = "switchport trunk native vlan {0}".format(
                proposed.get("native_vlan")
            )
            commands.append(command)

    if commands:
        commands.insert(0, "interface " + name)
    return commands


def is_switchport_default(existing):
    """Determines if switchport has a default config based on mode
    Args:
        existing (dict): existing switchport configuration from Ansible mod
    Returns:
        boolean: True if switchport has OOB Layer 2 config, i.e.
           vlan 1 and trunk all and mode is access
    """

    c1 = str(existing["access_vlan"]) == "1"
    c2 = str(existing["native_vlan"]) == "1"
    c3 = existing["trunk_vlans"] == "1-4094"
    c4 = existing["mode"] == "access"

    default = c1 and c2 and c3 and c4

    return default


def default_switchport_config(name):
    commands = []
    commands.append("interface " + name)
    commands.append("switchport mode access")
    commands.append("switch access vlan 1")
    commands.append("switchport trunk native vlan 1")
    commands.append("switchport trunk allowed vlan all")
    return commands


def vlan_range_to_list(vlans):
    result = []
    if vlans:
        for part in vlans.split(","):
            if part == "none":
                break
            if "-" in part:
                a, b = part.split("-")
                a, b = int(a), int(b)
                result.extend(range(a, b + 1))
            else:
                a = int(part)
                result.append(a)
        return numerical_sort(result)
    return result


def get_list_of_vlans(module):

    command = "show vlan | json"
    vlan_list = []

    try:
        body = run_commands(module, [command])[0]
        vlan_table = body["TABLE_vlanbrief"]["ROW_vlanbrief"]
    except (KeyError, AttributeError, IndexError):
        return []

    if isinstance(vlan_table, list):
        for vlan in vlan_table:
            vlan_list.append(str(vlan["vlanshowbr-vlanid-utf"]))
    else:
        vlan_list.append("1")

    return vlan_list


def numerical_sort(string_int_list):
    """Sorts list of strings/integers that are digits in numerical order.
    """

    as_int_list = []
    as_str_list = []
    for vlan in string_int_list:
        as_int_list.append(int(vlan))
    as_int_list.sort()
    for vlan in as_int_list:
        as_str_list.append(str(vlan))
    return as_str_list


def apply_key_map(key_map, table):
    new_dict = {}
    for key, value in table.items():
        new_key = key_map.get(key)
        if new_key:
            new_dict[new_key] = str(value)
    return new_dict


def apply_value_map(value_map, resource):
    for key, value in value_map.items():
        resource[key] = value[resource.get(key)]
    return resource


def flatten_list(command_lists):
    flat_command_list = []
    for command in command_lists:
        if isinstance(command, list):
            flat_command_list.extend(command)
        else:
            flat_command_list.append(command)
    return flat_command_list


def map_params_to_obj(module):
    obj = []
    aggregate = module.params.get("aggregate")
    if aggregate:
        for item in aggregate:
            for key in item:
                if item.get(key) is None:
                    item[key] = module.params[key]

            d = item.copy()
            obj.append(d)
    else:
        obj.append(
            {
                "name": module.params["name"],
                "mode": module.params["mode"],
                "access_vlan": module.params["access_vlan"],
                "native_vlan": module.params["native_vlan"],
                "trunk_vlans": module.params["trunk_vlans"],
                "trunk_allowed_vlans": module.params["trunk_allowed_vlans"],
                "state": module.params["state"],
            }
        )

    return obj


def main():
    """ main entry point for module execution
    """
    element_spec = dict(
        name=dict(type="str", aliases=["interface"]),
        mode=dict(choices=["access", "trunk"]),
        access_vlan=dict(type="str"),
        native_vlan=dict(type="str"),
        trunk_vlans=dict(type="str", aliases=["trunk_add_vlans"]),
        trunk_allowed_vlans=dict(type="str"),
        state=dict(
            choices=["absent", "present", "unconfigured"], default="present"
        ),
    )

    aggregate_spec = deepcopy(element_spec)

    # remove default in aggregate spec, to handle common arguments
    remove_default_spec(aggregate_spec)

    argument_spec = dict(
        aggregate=dict(type="list", elements="dict", options=aggregate_spec)
    )

    argument_spec.update(element_spec)
    argument_spec.update(nxos_argument_spec)

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=[
            ["access_vlan", "trunk_vlans"],
            ["access_vlan", "native_vlan"],
            ["access_vlan", "trunk_allowed_vlans"],
        ],
        supports_check_mode=True,
    )

    warnings = list()
    commands = []
    result = {"changed": False}
    if warnings:
        result["warnings"] = warnings

    want = map_params_to_obj(module)
    for w in want:
        name = w["name"]
        mode = w["mode"]
        access_vlan = w["access_vlan"]
        state = w["state"]
        trunk_vlans = w["trunk_vlans"]
        native_vlan = w["native_vlan"]
        trunk_allowed_vlans = w["trunk_allowed_vlans"]

        args = dict(
            name=name,
            mode=mode,
            access_vlan=access_vlan,
            native_vlan=native_vlan,
            trunk_vlans=trunk_vlans,
            trunk_allowed_vlans=trunk_allowed_vlans,
        )

        proposed = dict((k, v) for k, v in args.items() if v is not None)

        name = name.lower()

        if mode == "access" and state == "present" and not access_vlan:
            module.fail_json(
                msg="access_vlan param is required when mode=access && state=present"
            )

        if mode == "trunk" and access_vlan:
            module.fail_json(
                msg="access_vlan param not supported when using mode=trunk"
            )

        current_mode = get_interface_mode(name, module)

        # Current mode will return layer3, layer2, or unknown
        if current_mode == "unknown" or current_mode == "layer3":
            module.fail_json(
                msg="Ensure interface is configured to be a L2"
                "\nport first before using this module. You can use"
                "\nthe nxos_interface module for this."
            )

        if interface_is_portchannel(name, module):
            module.fail_json(
                msg="Cannot change L2 config on physical "
                "\nport because it is in a portchannel. "
                "\nYou should update the portchannel config."
            )

        # existing will never be null for Eth intfs as there is always a default
        existing = get_switchport(name, module)

        # Safeguard check
        # If there isn't an existing, something is wrong per previous comment
        if not existing:
            module.fail_json(
                msg="Make sure you are using the FULL interface name"
            )

        if trunk_vlans or trunk_allowed_vlans:
            if trunk_vlans:
                trunk_vlans_list = vlan_range_to_list(trunk_vlans)
            elif trunk_allowed_vlans:
                trunk_vlans_list = vlan_range_to_list(trunk_allowed_vlans)
                proposed["allowed"] = True

            existing_trunks_list = vlan_range_to_list(
                (existing["trunk_vlans"])
            )

            existing["trunk_vlans_list"] = existing_trunks_list
            proposed["trunk_vlans_list"] = trunk_vlans_list

        current_vlans = get_list_of_vlans(module)

        if state == "present":
            if access_vlan and access_vlan not in current_vlans:
                module.fail_json(
                    msg="You are trying to configure a VLAN"
                    " on an interface that\ndoes not exist on the "
                    " switch yet!",
                    vlan=access_vlan,
                )
            elif native_vlan and native_vlan not in current_vlans:
                module.fail_json(
                    msg="You are trying to configure a VLAN"
                    " on an interface that\ndoes not exist on the "
                    " switch yet!",
                    vlan=native_vlan,
                )
            else:
                command = get_switchport_config_commands(
                    name, existing, proposed, module
                )
                commands.append(command)
        elif state == "unconfigured":
            is_default = is_switchport_default(existing)
            if not is_default:
                command = default_switchport_config(name)
                commands.append(command)
        elif state == "absent":
            command = remove_switchport_config_commands(
                name, existing, proposed, module
            )
            commands.append(command)

        if trunk_vlans or trunk_allowed_vlans:
            existing.pop("trunk_vlans_list")
            proposed.pop("trunk_vlans_list")

    cmds = flatten_list(commands)
    if cmds:
        if module.check_mode:
            module.exit_json(changed=True, commands=cmds)
        else:
            result["changed"] = True
            load_config(module, cmds)
            if "configure" in cmds:
                cmds.pop(0)

    result["commands"] = cmds
    result["warnings"] = warnings

    module.exit_json(**result)


if __name__ == "__main__":
    main()



# Copyright 2018 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <https://www.gnu.org/licenses/>.

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import os
import json
from ansible_collections.fortinet.fortios.plugins.module_utils.fortimanager.fortimanager import FortiManagerHandler
import pytest

try:
    from ansible_collections.community.network.plugins.modules.network.fortimanager import fmgr_secprof_proxy
except ImportError:
    pytest.skip("Could not load required modules for testing", allow_module_level=True)


def load_fixtures():
    fixture_path = os.path.join(os.path.dirname(__file__), 'fixtures') + "/{filename}.json".format(
        filename=os.path.splitext(os.path.basename(__file__))[0])
    try:
        with open(fixture_path, "r") as fixture_file:
            fixture_data = json.load(fixture_file)
    except IOError:
        return []
    return [fixture_data]


@pytest.fixture(autouse=True)
def module_mock(mocker):
    connection_class_mock = mocker.patch('ansible.module_utils.basic.AnsibleModule')
    return connection_class_mock


@pytest.fixture(autouse=True)
def connection_mock(mocker):
    connection_class_mock = mocker.patch('ansible_collections.community.network.plugins.modules.network.fortimanager.fmgr_secprof_proxy.Connection')
    return connection_class_mock


@pytest.fixture(scope="function", params=load_fixtures())
def fixture_data(request):
    func_name = request.function.__name__.replace("test_", "")
    return request.param.get(func_name, None)


fmg_instance = FortiManagerHandler(connection_mock, module_mock)


def test_fmgr_web_proxy_profile_modify(fixture_data, mocker):
    mocker.patch('ansible_collections.fortinet.fortios.plugins.module_utils.fortimanager.fortimanager.FortiManagerHandler.process_request',
                 side_effect=fixture_data)

    # Test using fixture 1 #
    output = fmgr_secprof_proxy.fmgr_web_proxy_profile_modify(fmg_instance, fixture_data[0]['paramgram_used'])
    assert output['raw_response']['status']['code'] == 0
    # Test using fixture 2 #
    output = fmgr_secprof_proxy.fmgr_web_proxy_profile_modify(fmg_instance, fixture_data[1]['paramgram_used'])
    assert output['raw_response']['status']['code'] == 0



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
---
module: ce_bfd_global
short_description: Manages BFD global configuration on HUAWEI CloudEngine devices.
description:
    - Manages BFD global configuration on HUAWEI CloudEngine devices.
author: QijunPan (@QijunPan)
notes:
  - This module requires the netconf system service be enabled on the remote device being managed.
  - Recommended connection is C(netconf).
  - This module also works with C(local) connections for legacy playbooks.
options:
    bfd_enable:
        description:
            - Enables the global Bidirectional Forwarding Detection (BFD) function.
        choices: ['enable', 'disable']
    default_ip:
        description:
            - Specifies the default multicast IP address.
              The value ranges from 224.0.0.107 to 224.0.0.250.
    tos_exp_dynamic:
        description:
            - Indicates the priority of BFD control packets for dynamic BFD sessions.
              The value is an integer ranging from 0 to 7.
              The default priority is 7, which is the highest priority of BFD control packets.
    tos_exp_static:
        description:
            - Indicates the priority of BFD control packets for static BFD sessions.
              The value is an integer ranging from 0 to 7.
              The default priority is 7, which is the highest priority of BFD control packets.
    damp_init_wait_time:
        description:
            - Specifies an initial flapping suppression time for a BFD session.
              The value is an integer ranging from 1 to 3600000, in milliseconds.
              The default value is 2000.
    damp_max_wait_time:
        description:
            - Specifies a maximum flapping suppression time for a BFD session.
              The value is an integer ranging from 1 to 3600000, in milliseconds.
              The default value is 15000.
    damp_second_wait_time:
        description:
            - Specifies a secondary flapping suppression time for a BFD session.
              The value is an integer ranging from 1 to 3600000, in milliseconds.
              The default value is 5000.
    delay_up_time:
        description:
            - Specifies the delay before a BFD session becomes Up.
              The value is an integer ranging from 1 to 600, in seconds.
              The default value is 0, indicating that a BFD session immediately becomes Up.
    state:
        description:
            - Determines whether the config should be present or not on the device.
        default: present
        choices: ['present', 'absent']
'''

EXAMPLES = '''
- name: Bfd global module test
  hosts: cloudengine
  connection: local
  gather_facts: no
  vars:
    cli:
      host: "{{ inventory_hostname }}"
      port: "{{ ansible_ssh_port }}"
      username: "{{ username }}"
      password: "{{ password }}"
      transport: cli

  tasks:
  - name: Enable the global BFD function
    community.network.ce_bfd_global:
      bfd_enable: enable
      provider: '{{ cli }}'

  - name: Set the default multicast IP address to 224.0.0.150
    community.network.ce_bfd_global:
      bfd_enable: enable
      default_ip: 224.0.0.150
      state: present
      provider: '{{ cli }}'

  - name: Set the priority of BFD control packets for dynamic and static BFD sessions
    community.network.ce_bfd_global:
      bfd_enable: enable
      tos_exp_dynamic: 5
      tos_exp_static: 6
      state: present
      provider: '{{ cli }}'

  - name: Disable the global BFD function
    community.network.ce_bfd_global:
      bfd_enable: disable
      provider: '{{ cli }}'
'''

RETURN = '''
proposed:
    description: k/v pairs of parameters passed into module
    returned: verbose mode
    type: dict
    sample: {
        "bfd_enalbe": "enable",
        "damp_init_wait_time": null,
        "damp_max_wait_time": null,
        "damp_second_wait_time": null,
        "default_ip": null,
        "delayUpTimer": null,
        "state": "present",
        "tos_exp_dynamic": null,
        "tos_exp_static": null
    }
existing:
    description: k/v pairs of existing configuration
    returned: verbose mode
    type: dict
    sample: {
        "global": {
            "bfdEnable": "false",
            "dampInitWaitTime": "2000",
            "dampMaxWaitTime": "12000",
            "dampSecondWaitTime": "5000",
            "defaultIp": "224.0.0.184",
            "delayUpTimer": null,
            "tosExp": "7",
            "tosExpStatic": "7"
        }
    }
end_state:
    description: k/v pairs of configuration after module execution
    returned: verbose mode
    type: dict
    sample: {
        "global": {
            "bfdEnable": "true",
            "dampInitWaitTime": "2000",
            "dampMaxWaitTime": "12000",
            "dampSecondWaitTime": "5000",
            "defaultIp": "224.0.0.184",
            "delayUpTimer": null,
            "tosExp": "7",
            "tosExpStatic": "7"
        }
    }
updates:
    description: commands sent to the device
    returned: always
    type: list
    sample: [ "bfd" ]
changed:
    description: check to see if a change was made on the device
    returned: always
    type: bool
    sample: true
'''

import sys
import socket
from xml.etree import ElementTree
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.network.plugins.module_utils.network.cloudengine.ce import get_nc_config, set_nc_config, ce_argument_spec, check_ip_addr

CE_NC_GET_BFD = """
    <filter type="subtree">
      <bfd xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
      %s
      </bfd>
    </filter>
"""

CE_NC_GET_BFD_GLB = """
        <bfdSchGlobal>
          <bfdEnable></bfdEnable>
          <defaultIp></defaultIp>
          <tosExp></tosExp>
          <tosExpStatic></tosExpStatic>
          <dampInitWaitTime></dampInitWaitTime>
          <dampMaxWaitTime></dampMaxWaitTime>
          <dampSecondWaitTime></dampSecondWaitTime>
          <delayUpTimer></delayUpTimer>
        </bfdSchGlobal>
"""


def check_default_ip(ipaddr):
    """check the default multicast IP address"""

    # The value ranges from 224.0.0.107 to 224.0.0.250
    if not check_ip_addr(ipaddr):
        return False

    if ipaddr.count(".") != 3:
        return False

    ips = ipaddr.split(".")
    if ips[0] != "224" or ips[1] != "0" or ips[2] != "0":
        return False

    if not ips[3].isdigit() or int(ips[3]) < 107 or int(ips[3]) > 250:
        return False

    return True


class BfdGlobal(object):
    """Manages BFD Global"""

    def __init__(self, argument_spec):
        self.spec = argument_spec
        self.module = None
        self.__init_module__()

        # module input info
        self.bfd_enable = self.module.params['bfd_enable']
        self.default_ip = self.module.params['default_ip']
        self.tos_exp_dynamic = self.module.params['tos_exp_dynamic']
        self.tos_exp_static = self.module.params['tos_exp_static']
        self.damp_init_wait_time = self.module.params['damp_init_wait_time']
        self.damp_max_wait_time = self.module.params['damp_max_wait_time']
        self.damp_second_wait_time = self.module.params['damp_second_wait_time']
        self.delay_up_time = self.module.params['delay_up_time']
        self.state = self.module.params['state']

        # host info
        self.host = self.module.params['host']
        self.username = self.module.params['username']
        self.port = self.module.params['port']

        # state
        self.changed = False
        self.bfd_dict = dict()
        self.updates_cmd = list()
        self.commands = list()
        self.results = dict()
        self.proposed = dict()
        self.existing = dict()
        self.end_state = dict()

    def __init_module__(self):
        """init module"""

        required_together = [('damp_init_wait_time', 'damp_max_wait_time', 'damp_second_wait_time')]
        self.module = AnsibleModule(argument_spec=self.spec,
                                    required_together=required_together,
                                    supports_check_mode=True)

    def get_bfd_dict(self):
        """bfd config dict"""

        bfd_dict = dict()
        bfd_dict["global"] = dict()
        conf_str = CE_NC_GET_BFD % CE_NC_GET_BFD_GLB

        xml_str = get_nc_config(self.module, conf_str)
        if "<data/>" in xml_str:
            return bfd_dict

        xml_str = xml_str.replace('\r', '').replace('\n', '').\
            replace('xmlns="urn:ietf:params:xml:ns:netconf:base:1.0"', "").\
            replace('xmlns="http://www.huawei.com/netconf/vrp"', "")
        root = ElementTree.fromstring(xml_str)

        # get bfd global info
        glb = root.find("bfd/bfdSchGlobal")
        if glb:
            for attr in glb:
                if attr.text is not None:
                    bfd_dict["global"][attr.tag] = attr.text

        return bfd_dict

    def config_global(self):
        """configures bfd global params"""

        xml_str = ""
        damp_chg = False

        # bfd_enable
        if self.bfd_enable:
            if bool(self.bfd_dict["global"].get("bfdEnable", "false") == "true") != bool(self.bfd_enable == "enable"):
                if self.bfd_enable == "enable":
                    xml_str = "<bfdEnable>true</bfdEnable>"
                    self.updates_cmd.append("bfd")
                else:
                    xml_str = "<bfdEnable>false</bfdEnable>"
                    self.updates_cmd.append("undo bfd")

        # get bfd end state
        bfd_state = "disable"
        if self.bfd_enable:
            bfd_state = self.bfd_enable
        elif self.bfd_dict["global"].get("bfdEnable", "false") == "true":
            bfd_state = "enable"

        # default_ip
        if self.default_ip:
            if bfd_state == "enable":
                if self.state == "present" and self.default_ip != self.bfd_dict["global"].get("defaultIp"):
                    xml_str += "<defaultIp>%s</defaultIp>" % self.default_ip
                    if "bfd" not in self.updates_cmd:
                        self.updates_cmd.append("bfd")
                    self.updates_cmd.append("default-ip-address %s" % self.default_ip)
                elif self.state == "absent" and self.default_ip == self.bfd_dict["global"].get("defaultIp"):
                    xml_str += "<defaultIp/>"
                    if "bfd" not in self.updates_cmd:
                        self.updates_cmd.append("bfd")
                    self.updates_cmd.append("undo default-ip-address")

        # tos_exp_dynamic
        if self.tos_exp_dynamic is not None:
            if bfd_state == "enable":
                if self.state == "present" and self.tos_exp_dynamic != int(self.bfd_dict["global"].get("tosExp", "7")):
                    xml_str += "<tosExp>%s</tosExp>" % self.tos_exp_dynamic
                    if "bfd" not in self.updates_cmd:
                        self.updates_cmd.append("bfd")
                    self.updates_cmd.append("tos-exp %s dynamic" % self.tos_exp_dynamic)
                elif self.state == "absent" and self.tos_exp_dynamic == int(self.bfd_dict["global"].get("tosExp", "7")):
                    xml_str += "<tosExp/>"
                    if "bfd" not in self.updates_cmd:
                        self.updates_cmd.append("bfd")
                    self.updates_cmd.append("undo tos-exp dynamic")

        # tos_exp_static
        if self.tos_exp_static is not None:
            if bfd_state == "enable":
                if self.state == "present" \
                        and self.tos_exp_static != int(self.bfd_dict["global"].get("tosExpStatic", "7")):
                    xml_str += "<tosExpStatic>%s</tosExpStatic>" % self.tos_exp_static
                    if "bfd" not in self.updates_cmd:
                        self.updates_cmd.append("bfd")
                    self.updates_cmd.append("tos-exp %s static" % self.tos_exp_static)
                elif self.state == "absent" \
                        and self.tos_exp_static == int(self.bfd_dict["global"].get("tosExpStatic", "7")):
                    xml_str += "<tosExpStatic/>"
                    if "bfd" not in self.updates_cmd:
                        self.updates_cmd.append("bfd")
                    self.updates_cmd.append("undo tos-exp static")

        # delay_up_time
        if self.delay_up_time is not None:
            if bfd_state == "enable":
                delay_time = self.bfd_dict["global"].get("delayUpTimer", "0")
                if not delay_time or not delay_time.isdigit():
                    delay_time = "0"
                if self.state == "present" \
                        and self.delay_up_time != int(delay_time):
                    xml_str += "<delayUpTimer>%s</delayUpTimer>" % self.delay_up_time
                    if "bfd" not in self.updates_cmd:
                        self.updates_cmd.append("bfd")
                    self.updates_cmd.append("delay-up %s" % self.delay_up_time)
                elif self.state == "absent" \
                        and self.delay_up_time == int(delay_time):
                    xml_str += "<delayUpTimer/>"
                    if "bfd" not in self.updates_cmd:
                        self.updates_cmd.append("bfd")
                    self.updates_cmd.append("undo delay-up")

        # damp_init_wait_time damp_max_wait_time damp_second_wait_time
        if self.damp_init_wait_time is not None and self.damp_second_wait_time is not None \
                and self.damp_second_wait_time is not None:
            if bfd_state == "enable":
                if self.state == "present":
                    if self.damp_max_wait_time != int(self.bfd_dict["global"].get("dampMaxWaitTime", "2000")):
                        xml_str += "<dampMaxWaitTime>%s</dampMaxWaitTime>" % self.damp_max_wait_time
                        damp_chg = True
                    if self.damp_init_wait_time != int(self.bfd_dict["global"].get("dampInitWaitTime", "12000")):
                        xml_str += "<dampInitWaitTime>%s</dampInitWaitTime>" % self.damp_init_wait_time
                        damp_chg = True
                    if self.damp_second_wait_time != int(self.bfd_dict["global"].get("dampSecondWaitTime", "5000")):
                        xml_str += "<dampSecondWaitTime>%s</dampSecondWaitTime>" % self.damp_second_wait_time
                        damp_chg = True
                    if damp_chg:
                        if "bfd" not in self.updates_cmd:
                            self.updates_cmd.append("bfd")
                        self.updates_cmd.append("dampening timer-interval maximum %s initial %s secondary %s" % (
                            self.damp_max_wait_time, self.damp_init_wait_time, self.damp_second_wait_time))
                else:
                    damp_chg = True
                    if self.damp_max_wait_time != int(self.bfd_dict["global"].get("dampMaxWaitTime", "2000")):
                        damp_chg = False
                    if self.damp_init_wait_time != int(self.bfd_dict["global"].get("dampInitWaitTime", "12000")):
                        damp_chg = False
                    if self.damp_second_wait_time != int(self.bfd_dict["global"].get("dampSecondWaitTime", "5000")):
                        damp_chg = False

                    if damp_chg:
                        xml_str += "<dampMaxWaitTime/><dampInitWaitTime/><dampSecondWaitTime/>"
                        if "bfd" not in self.updates_cmd:
                            self.updates_cmd.append("bfd")
                        self.updates_cmd.append("undo dampening timer-interval maximum %s initial %s secondary %s" % (
                            self.damp_max_wait_time, self.damp_init_wait_time, self.damp_second_wait_time))
        if xml_str:
            return '<bfdSchGlobal operation="merge">' + xml_str + '</bfdSchGlobal>'
        else:
            return ""

    def netconf_load_config(self, xml_str):
        """load bfd config by netconf"""

        if not xml_str:
            return

        xml_cfg = """
            <config>
            <bfd xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
            %s
            </bfd>
            </config>""" % xml_str
        set_nc_config(self.module, xml_cfg)
        self.changed = True

    def check_params(self):
        """Check all input params"""

        # check default_ip
        if self.default_ip:
            if not check_default_ip(self.default_ip):
                self.module.fail_json(msg="Error: Default ip is invalid.")

        # check tos_exp_dynamic
        if self.tos_exp_dynamic is not None:
            if self.tos_exp_dynamic < 0 or self.tos_exp_dynamic > 7:
                self.module.fail_json(msg="Error: Session tos_exp_dynamic is not ranges from 0 to 7.")

        # check tos_exp_static
        if self.tos_exp_static is not None:
            if self.tos_exp_static < 0 or self.tos_exp_static > 7:
                self.module.fail_json(msg="Error: Session tos_exp_static is not ranges from 0 to 7.")

        # check damp_init_wait_time
        if self.damp_init_wait_time is not None:
            if self.damp_init_wait_time < 1 or self.damp_init_wait_time > 3600000:
                self.module.fail_json(msg="Error: Session damp_init_wait_time is not ranges from 1 to 3600000.")

        # check damp_max_wait_time
        if self.damp_max_wait_time is not None:
            if self.damp_max_wait_time < 1 or self.damp_max_wait_time > 3600000:
                self.module.fail_json(msg="Error: Session damp_max_wait_time is not ranges from 1 to 3600000.")

        # check damp_second_wait_time
        if self.damp_second_wait_time is not None:
            if self.damp_second_wait_time < 1 or self.damp_second_wait_time > 3600000:
                self.module.fail_json(msg="Error: Session damp_second_wait_time is not ranges from 1 to 3600000.")

        # check delay_up_time
        if self.delay_up_time is not None:
            if self.delay_up_time < 1 or self.delay_up_time > 600:
                self.module.fail_json(msg="Error: Session delay_up_time is not ranges from 1 to 600.")

    def get_proposed(self):
        """get proposed info"""

        self.proposed["bfd_enalbe"] = self.bfd_enable
        self.proposed["default_ip"] = self.default_ip
        self.proposed["tos_exp_dynamic"] = self.tos_exp_dynamic
        self.proposed["tos_exp_static"] = self.tos_exp_static
        self.proposed["damp_init_wait_time"] = self.damp_init_wait_time
        self.proposed["damp_max_wait_time"] = self.damp_max_wait_time
        self.proposed["damp_second_wait_time"] = self.damp_second_wait_time
        self.proposed["delay_up_time"] = self.delay_up_time
        self.proposed["state"] = self.state

    def get_existing(self):
        """get existing info"""

        if not self.bfd_dict:
            return

        self.existing["global"] = self.bfd_dict.get("global")

    def get_end_state(self):
        """get end state info"""

        bfd_dict = self.get_bfd_dict()
        if not bfd_dict:
            return

        self.end_state["global"] = bfd_dict.get("global")
        if self.existing == self.end_state:
            self.changed = False

    def work(self):
        """worker"""

        self.check_params()
        self.bfd_dict = self.get_bfd_dict()
        self.get_existing()
        self.get_proposed()

        # deal present or absent
        xml_str = self.config_global()

        # update to device
        if xml_str:
            self.netconf_load_config(xml_str)
            self.changed = True

        self.get_end_state()
        self.results['changed'] = self.changed
        self.results['proposed'] = self.proposed
        self.results['existing'] = self.existing
        self.results['end_state'] = self.end_state
        if self.changed:
            self.results['updates'] = self.updates_cmd
        else:
            self.results['updates'] = list()

        self.module.exit_json(**self.results)


def main():
    """Module main"""

    argument_spec = dict(
        bfd_enable=dict(required=False, type='str', choices=['enable', 'disable']),
        default_ip=dict(required=False, type='str'),
        tos_exp_dynamic=dict(required=False, type='int'),
        tos_exp_static=dict(required=False, type='int'),
        damp_init_wait_time=dict(required=False, type='int'),
        damp_max_wait_time=dict(required=False, type='int'),
        damp_second_wait_time=dict(required=False, type='int'),
        delay_up_time=dict(required=False, type='int'),
        state=dict(required=False, default='present', choices=['present', 'absent'])
    )

    argument_spec.update(ce_argument_spec)
    module = BfdGlobal(argument_spec)
    module.work()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2020, Kevin Breit (@kbreit) <kevin.breit@kevinbreit.net>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}

DOCUMENTATION = r'''
---
module: meraki_ms_link_aggregation
short_description: Manage link aggregations on MS switches
version_added: "1.2.0"
description:
- Allows for management of MS switch link aggregations in a Meraki environment.
notes:
- Switch profile ports are not supported in this module.
options:
    state:
        description:
        - Specifies whether SNMP information should be queried or modified.
        type: str
        choices: [ absent, query, present ]
        default: present
    net_name:
        description:
        - Name of network.
        type: str
    net_id:
        description:
        - ID of network.
        type: str
    lag_id:
        description:
        - ID of lag to query or modify.
        type: str
    switch_ports:
        description:
        - List of switchports to include in link aggregation.
        type: list
        elements: dict
        suboptions:
            serial:
                description:
                - Serial number of switch to own link aggregation.
                type: str
            port_id:
                description:
                - Port number which should be included in link aggregation.
                type: str
author:
- Kevin Breit (@kbreit)
extends_documentation_fragment: cisco.meraki.meraki
'''

EXAMPLES = r'''
- name: Create LAG
  meraki_ms_link_aggregation:
    auth_key: '{{auth_key}}'
    state: present
    org_name: '{{test_org_name}}'
    net_name: '{{test_switch_net_name}}'
    switch_ports:
      - serial: '{{serial_switch}}'
        port_id: "1"
      - serial: '{{serial_switch}}'
        port_id: "2"
  delegate_to: localhost

- name: Update LAG
  meraki_ms_link_aggregation:
    auth_key: '{{auth_key}}'
    state: present
    org_name: '{{test_org_name}}'
    net_name: '{{test_switch_net_name}}'
    lag_id: '{{lag_id}}'
    switch_ports:
      - serial: '{{serial_switch}}'
        port_id: "1"
      - serial: '{{serial_switch}}'
        port_id: "2"
      - serial: '{{serial_switch}}'
        port_id: "3"
      - serial: '{{serial_switch}}'
        port_id: "4"
  delegate_to: localhost
'''

RETURN = r'''
data:
  description: List of aggregated links.
  returned: success
  type: complex
  contains:
      id:
          description:
            - ID of link aggregation.
          returned: success
          type: str
          sample: "MTK3M4A2ZDdfM3=="
      switch_ports:
          description:
            - List of switch ports to be included in link aggregation.
          returned: success
          type: complex
          contains:
              port_id:
                description:
                  - Port number.
                type: str
                returned: success
                sample: "1"
              serial:
                description:
                  - Serial number of switch on which port resides.
                type: str
                returned: success
                sample: "ABCD-1234-WXYZ"
'''

from ansible.module_utils.basic import AnsibleModule, json
from ansible_collections.cisco.meraki.plugins.module_utils.network.meraki.meraki import MerakiModule, meraki_argument_spec


def get_lags(meraki, net_id):
    path = meraki.construct_path('get_all', net_id=net_id)
    return meraki.request(path, method='GET')


def is_lag_valid(lags, lag_id):
    for lag in lags:
        if lag['id'] == lag_id:
            return lag
    return False


def construct_payload(meraki):
    payload = dict()
    if meraki.params['switch_ports'] is not None:
        payload['switchPorts'] = []
        for port in meraki.params['switch_ports']:
            port_config = {'serial': port['serial'],
                           'portId': port['port_id'],
                           }
            payload['switchPorts'].append(port_config)
    # if meraki.params['switch_profile_ports'] is not None:
    #     payload['switchProfilePorts'] = []
    #     for port in meraki.params['switch_profile_ports']:
    #         port_config = {'profile': port['profile'],
    #                        'portId': port['port_id'],
    #                        }
    #         payload['switchProfilePorts'].append(port_config)
    return payload


def main():

    # define the available arguments/parameters that a user can pass to
    # the module

    switch_ports_args = dict(serial=dict(type='str'),
                             port_id=dict(type='str'),
                             )

    # switch_profile_ports_args = dict(profile=dict(type='str'),
    #                                  port_id=dict(type='str'),
    #                                  )

    argument_spec = meraki_argument_spec()
    argument_spec.update(state=dict(type='str', choices=['absent', 'present', 'query'], default='present'),
                         org_name=dict(type='str', aliases=['organization']),
                         org_id=dict(type='str'),
                         net_name=dict(type='str'),
                         net_id=dict(type='str'),
                         lag_id=dict(type='str'),
                         switch_ports=dict(type='list', default=None, elements='dict', options=switch_ports_args),
                         # switch_profile_ports=dict(type='list', default=None, elements='dict', options=switch_profile_ports_args),
                         )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True,
                           )
    meraki = MerakiModule(module, function='ms_link_aggregation')
    meraki.params['follow_redirects'] = 'all'

    query_urls = {'ms_link_aggregation': '/networks/{net_id}/switch/linkAggregations'}
    create_url = {'ms_link_aggregation': '/networks/{net_id}/switch/linkAggregations'}
    update_url = {'ms_link_aggregation': '/networks/{net_id}/switch/linkAggregations/{lag_id}'}
    delete_url = {'ms_link_aggregation': '/networks/{net_id}/switch/linkAggregations/{lag_id}'}

    meraki.url_catalog['get_all'].update(query_urls)
    meraki.url_catalog['create'] = create_url
    meraki.url_catalog['update'] = update_url
    meraki.url_catalog['delete'] = delete_url

    payload = None

    # execute checks for argument completeness
    # manipulate or modify the state as needed (this is going to be the
    # part where your module will do what it needs to do)
    org_id = meraki.params['org_id']
    if org_id is None:
        org_id = meraki.get_org_id(meraki.params['org_name'])
    net_id = meraki.params['net_id']
    if net_id is None:
        nets = meraki.get_nets(org_id=org_id)
        net_id = meraki.get_net_id(org_id, meraki.params['net_name'], data=nets)

    if meraki.params['state'] == 'query':
        path = meraki.construct_path('get_all', net_id=net_id)
        response = meraki.request(path, method='GET')
        meraki.result['data'] = response
        meraki.exit_json(**meraki.result)
    elif meraki.params['state'] == 'present':
        if meraki.params['lag_id'] is not None:  # Need to update
            lag = is_lag_valid(get_lags(meraki, net_id), meraki.params['lag_id'])
            if lag is not False:  # Lag ID is valid
                payload = construct_payload(meraki)
                if meraki.is_update_required(lag, payload) is True:
                    path = meraki.construct_path('update', net_id=net_id, custom={'lag_id': meraki.params['lag_id']})
                    response = meraki.request(path, method='PUT', payload=json.dumps(payload))
                    meraki.result['changed'] = True
                    meraki.result['data'] = response
                else:
                    meraki.result['data'] = lag
            else:
                meraki.fail_json("Provided lag_id is not valid.")
        else:
            path = meraki.construct_path('create', net_id=net_id)
            payload = construct_payload(meraki)
            response = meraki.request(path, method='POST', payload=json.dumps(payload))
            meraki.result['changed'] = True
            meraki.result['data'] = response
        meraki.exit_json(**meraki.result)
    elif meraki.params['state'] == 'absent':
        path = meraki.construct_path('delete', net_id=net_id, custom={'lag_id': meraki.params['lag_id']})
        response = meraki.request(path, method='DELETE')
        meraki.result['data'] = {}
        meraki.result['changed'] = True

    # in the event of a successful module execution, you will want to
    # simple AnsibleModule.exit_json(), passing the key/value results
    meraki.exit_json(**meraki.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# @author: Gaurav Rastogi (grastogi@avinetworks.com)
#          Eric Anderson (eanderson@avinetworks.com)
# module_check: supported
#
# Copyright: (c) 2017 Gaurav Rastogi, <grastogi@avinetworks.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

DOCUMENTATION = '''
---
module: avi_scheduler
author: Gaurav Rastogi (@grastogi23) <grastogi@avinetworks.com>

short_description: Module for setup of Scheduler Avi RESTful Object
description:
    - This module is used to configure Scheduler object
    - more examples at U(https://github.com/avinetworks/devops)
requirements: [ avisdk ]
options:
    state:
        description:
            - The state that should be applied on the entity.
        default: present
        choices: ["absent", "present"]
    avi_api_update_method:
        description:
            - Default method for object update is HTTP PUT.
            - Setting to patch will override that behavior to use HTTP PATCH.
        default: put
        choices: ["put", "patch"]
    avi_api_patch_op:
        description:
            - Patch operation to use when using avi_api_update_method as patch.
        choices: ["add", "replace", "delete"]
    backup_config_ref:
        description:
            - Backup configuration to be executed by this scheduler.
            - It is a reference to an object of type backupconfiguration.
    enabled:
        description:
            - Boolean flag to set enabled.
            - Default value when not specified in API or module is interpreted by Avi Controller as True.
        type: bool
    end_date_time:
        description:
            - Scheduler end date and time.
    frequency:
        description:
            - Frequency at which custom scheduler will run.
            - Allowed values are 0-60.
    frequency_unit:
        description:
            - Unit at which custom scheduler will run.
            - Enum options - SCHEDULER_FREQUENCY_UNIT_MIN, SCHEDULER_FREQUENCY_UNIT_HOUR, SCHEDULER_FREQUENCY_UNIT_DAY, SCHEDULER_FREQUENCY_UNIT_WEEK,
            - SCHEDULER_FREQUENCY_UNIT_MONTH.
    name:
        description:
            - Name of scheduler.
        required: true
    run_mode:
        description:
            - Scheduler run mode.
            - Enum options - RUN_MODE_PERIODIC, RUN_MODE_AT, RUN_MODE_NOW.
    run_script_ref:
        description:
            - Control script to be executed by this scheduler.
            - It is a reference to an object of type alertscriptconfig.
    scheduler_action:
        description:
            - Define scheduler action.
            - Enum options - SCHEDULER_ACTION_RUN_A_SCRIPT, SCHEDULER_ACTION_BACKUP.
            - Default value when not specified in API or module is interpreted by Avi Controller as SCHEDULER_ACTION_BACKUP.
    start_date_time:
        description:
            - Scheduler start date and time.
    tenant_ref:
        description:
            - It is a reference to an object of type tenant.
    url:
        description:
            - Avi controller URL of the object.
    uuid:
        description:
            - Unique object identifier of the object.
extends_documentation_fragment:
- community.network.avi

'''

EXAMPLES = """
- name: Example to create Scheduler object
  community.network.avi_scheduler:
    controller: 10.10.25.42
    username: admin
    password: something
    state: present
    name: sample_scheduler
"""

RETURN = '''
obj:
    description: Scheduler (api/scheduler) object
    returned: success, changed
    type: dict
'''

from ansible.module_utils.basic import AnsibleModule
try:
    from ansible_collections.community.network.plugins.module_utils.network.avi.avi import (
        avi_common_argument_spec, avi_ansible_api, HAS_AVI)
except ImportError:
    HAS_AVI = False


def main():
    argument_specs = dict(
        state=dict(default='present',
                   choices=['absent', 'present']),
        avi_api_update_method=dict(default='put',
                                   choices=['put', 'patch']),
        avi_api_patch_op=dict(choices=['add', 'replace', 'delete']),
        backup_config_ref=dict(type='str',),
        enabled=dict(type='bool',),
        end_date_time=dict(type='str',),
        frequency=dict(type='int',),
        frequency_unit=dict(type='str',),
        name=dict(type='str', required=True),
        run_mode=dict(type='str',),
        run_script_ref=dict(type='str',),
        scheduler_action=dict(type='str',),
        start_date_time=dict(type='str',),
        tenant_ref=dict(type='str',),
        url=dict(type='str',),
        uuid=dict(type='str',),
    )
    argument_specs.update(avi_common_argument_spec())
    module = AnsibleModule(
        argument_spec=argument_specs, supports_check_mode=True)
    if not HAS_AVI:
        return module.fail_json(msg=(
            'Avi python API SDK (avisdk>=17.1) or requests is not installed. '
            'For more details visit https://github.com/avinetworks/sdk.'))
    return avi_ansible_api(module, 'scheduler',
                           set([]))


if __name__ == '__main__':
    main()
